// ------------------------------------------------------------
// << C2DArray methods >>
// ------------------------------------------------------------
// ~C2DArray(): destructor
template<class T> C2DArray<T>::~C2DArray(){

	if(data){
		for(int y=0;y<h;y++)
			delete[] data[y];
	 
		delete[] data;
	}
	
	data = NULL;
	w = h = 0;
}

// create(): creates the array
template<class T> void C2DArray<T>::create(int width, int height){

	w = width;
	h = height;
	if(w<=0) w=1;
	if(h<=0) h=1;
	data = new T*[h];

	for(int y=0;y<h;y++)
		data[y] = new T[w];

}

// resize(): resizes the array
template<class T> void C2DArray<T>::resize(int new_width, int new_height, T value){

	T **temp;
	int x, y, oldw = w, oldh = h, m1, m2;

	// checking size
	if(new_width<=0) new_width = 1;
	if(new_height<=0) new_height = 1;

	// creating a dynamic array (temp) & copying (temp = (*this))
	temp = new T*[oldh];
	for(y=0;y<oldh;y++){
		temp[y] = new T[oldw];
		for(x=0;x<oldw;x++)
			temp[y][x] = data[y][x];
	}
	
	// realloc
	this->~C2DArray();
	create(new_width, new_height);
	clear(value);

	// copying ((*this) = temp)
	m1 = MIN(oldw,w);
	m2 = MIN(oldh,h);
	for(y=0;y<m2;y++)
		for(x=0;x<m1;x++)
			data[y][x] = temp[y][x];
		
	 // deleting the dynamic array (temp)
	for(y=0;y<oldh;y++)
		delete[] temp[y];
	delete[] temp;
}

// get()
template<class T> T C2DArray<T>::get(int x, int y){

	if(x<0) x=0; if(x>=w) x=w-1;
	if(y<0) y=0; if(y>=h) y=h-1;

	return data[y][x];

}

// pGet() - get pointer
template<class T> T* C2DArray<T>::pGet(int x, int y){

	if(x<0) x=0; if(x>=w) x=w-1;
	if(y<0) y=0; if(y>=h) y=h-1;

	return &data[y][x];

}

// set()
template<class T> void C2DArray<T>::set(int x, int y, T info){

	 if(x<0 || x>=w || y<0 || y>=h)
		return;
	 
	 if(data)
		data[y][x] = info;

}

// clear()
template<class T> void C2DArray<T>::clear(T value){
	
	static int x, y;
	
	if(!data)
		return;
	
	for(y=0;y<h;y++)
		for(x=0;x<w;x++)
			data[y][x] = value;
		
}
