/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include "Data.h"
 
 
 ///////////////////////////// Variables ////////////////////
 vector<BITMAP*>						Data::image;
 vector<CAnimation>						Data::animation;
 vector<CAnimation>						Data::superBgAnimation;
 vector<CAnimation>						Data::bgAnimation;
 vector<CAnimation>						Data::fgAnimation;
 vector<CEnemy>							Data::enemy;
 vector<CItem>							Data::item;
 vector<SAMPLE*>						Data::sample;
 vector<FMUSIC_MODULE*>					Data::music;
 vector<ALFONT_FONT*>					Data::font;
 vector<CLanguage>						Data::lang;
 vector<CPlayer>						Data::character;
 vector<CAnimation>						Data::mapTile;
 
 vector<string>							Data::sImage;
 vector<string>							Data::sAnimation;
 vector<string>							Data::sSuperBgAnimation;
 vector<string>							Data::sBgAnimation;
 vector<string>							Data::sFgAnimation;
 vector<string>							Data::sEnemy;
 vector<string>							Data::sItem;
 vector<string>							Data::sSample;
 vector<string>							Data::sMusic;
 vector<string>							Data::sFont;

 
 ///////////////////////////// Methods //////////////////////
 // ReadLists()
 void Data::ReadLists(){

	 int a;
	 string fname, line;
	 string list[] = { "images", "sprites", "superbgs", "bgs", "fgs", "enemies", "item", "samples", "musics", "fonts" };
	 ifstream fp[int(sizeof(list)/sizeof(string))];
	 
	 for(a=0; a<int(sizeof(list)/sizeof(string)); a++){
		 fname = "./data/lists/" + list[a];
		 if(!exists(fname.c_str())){
			 Game::logfile("Error: filelist \"%s\" doesn't exist",fname.c_str());
			 MsgBox("Error: filelist \"%s\" doesn't exist",fname.c_str());
			 Game::abort();
		 }
		 
		 fp[a].open(fname.c_str(), ios::in);
		 if(!fp[a].good()){
			 Game::logfile("Couldn't load filelist: \"%s\"",fname.c_str());
			 MsgBox("Cannot load filelist: \"%s\"",fname.c_str());
			 Game::abort();
		 }
		 
		 while(!fp[a].eof()){
			 getline(fp[a], line);
			 if(line == "") continue;
			 switch(a){
			 case 0: sImage.push_back("./data/images/" + line); break;
			 case 1: sAnimation.push_back("./data/sprites/" + line); break;
			 case 2: sSuperBgAnimation.push_back("./data/backgrounds/superbgs/" + line); break;
			 case 3: sBgAnimation.push_back("./data/backgrounds/bgs/" + line); break;
			 case 4: sFgAnimation.push_back("./data/backgrounds/fgs/" + line); break;
			 case 5: sEnemy.push_back("./data/enemies/" + line); break;
			 case 6: sItem.push_back("./data/item/" + line); break;
			 case 7: sSample.push_back("./data/samples/" + line); break;
			 case 8: sMusic.push_back("./data/musics/" + line); break;
			 case 9: sFont.push_back("./data/fonts/" + line); break;
			 }
		 }
		 
		 fp[a].close();
	 }
	 
 }
 
 // loadImages()
 void Data::loadImages(){
	 int a;
	 for(a=0;a<int(sImage.size());a++){
		 if(!exists(sImage[a].c_str())){
			 Game::logfile("Couldn't load image \"%s\" - file doesn't exist",sImage[a].c_str());
			 MsgBox("Cannot load image \"%s\" - file doesn't exist",sImage[a].c_str());
			 Game::abort();
		 }
		 image.push_back(load_bitmap(sImage[a].c_str(), NULL));
	 }
 }
	 
 // loadAnimations()
 void Data::loadAnimations(){
	 int a;
	 for(a=0;a<int(sAnimation.size());a++){
		 if(!exists(sAnimation[a].c_str())){
			 Game::logfile("Couldn't load animation \"%s\" - file doesn't exist",sAnimation[a].c_str());
			 MsgBox("Cannot load animation \"%s\" - file doesn't exist",sAnimation[a].c_str());
			 Game::abort();
		 }
		 animation.push_back(sAnimation[a]);
	 }
 }
 
 // loadBgAnimations()
 void Data::loadBgAnimations(){
	 int a;
	 for(a=0;a<int(sSuperBgAnimation.size());a++){
		 if(!exists(sSuperBgAnimation[a].c_str())){
			 Game::logfile("Couldn't load superBgAnimation \"%s\" - file not found", sSuperBgAnimation[a].c_str());
			 MsgBox("Cannot load superBgAnimation \"%s\" - file not found", sSuperBgAnimation[a].c_str());
			 Game::abort();
		 }
		 superBgAnimation.push_back(sSuperBgAnimation[a]);
	 }
	 
	 for(a=0;a<int(sBgAnimation.size());a++){
		 if(!exists(sBgAnimation[a].c_str())){
			 Game::logfile("Couldn't load bgAnimation \"%s\" - file not found", sBgAnimation[a].c_str());
			 MsgBox("Cannot load bgAnimation \"%s\" - file not found", sBgAnimation[a].c_str());
			 Game::abort();
		 }
		 bgAnimation.push_back(sBgAnimation[a]);
	 }
	 
	 for(a=0;a<int(sFgAnimation.size());a++){
		 if(!exists(sFgAnimation[a].c_str())){
			 Game::logfile("Couldn't load fgAnimation \"%s\" - file not found", sFgAnimation[a].c_str());
			 MsgBox("Cannot load fgAnimation \"%s\" - file not found", sFgAnimation[a].c_str());
			 Game::abort();
		 }
		 fgAnimation.push_back(sFgAnimation[a]);
	 }
 }
 
 // loadEnemies()
 void Data::loadEnemies(){
	 int a;
	 for(a=0;a<int(sEnemy.size());a++){
		 if(!exists(sEnemy[a].c_str())){
			 Game::logfile("Couldn't load enemy \"%s\" - file not found", sEnemy[a].c_str());
			 MsgBox("Cannot load enemy \"%s\" - file not found", sEnemy[a].c_str());
			 Game::abort();
		 }
		 enemy.push_back(sEnemy[a]);
	 }
 }
 
 // loadItems()
 void Data::loadItems(){
	 int a;
	 for(a=0;a<int(sItem.size());a++){
		 if(!exists(sItem[a].c_str())){
			 Game::logfile("Couldn't load item \"%s\" - file not found", sItem[a].c_str());
			 MsgBox("Cannot load item \"%s\" - file not found", sItem[a].c_str());
			 Game::abort();
		 }
		 item.push_back(sItem[a]);
	 }
 }
	 
 // loadSamples()
 void Data::loadSamples(){
	 int a;
	 for(a=0;a<int(sSample.size());a++){
		 if(!exists(sSample[a].c_str())){
			 Game::logfile("Couldn't load sample \"%s\" - file doesn't exist",sSample[a].c_str());
			 MsgBox("Cannot load sample \"%s\" - file doesn't exist",sSample[a].c_str());
			 Game::abort();
		 }
		 sample.push_back(load_sample(sSample[a].c_str()));
	 }
 }
	 
 // loadMusics()
 void Data::loadMusics(){
	 int a;
	 for(a=0;a<int(sMusic.size());a++){
		 if(!exists(sMusic[a].c_str())){
			 Game::logfile("Couldn't load music \"%s\" - file doesn't exist",sMusic[a].c_str());
			 MsgBox("Cannot load music \"%s\" - file doesn't exist",sMusic[a].c_str());
			 Game::abort();
		 }
		 music.push_back(FMUSIC_LoadSong(sMusic[a].c_str()));
	 }
 }
 
 // loadLanguages()
 void Data::loadLanguages(){
	 vector<string> filelist;
	 vector<string>::iterator it;
	 vector<CLanguage>::iterator itl;
	 string language_sign = SMWX2_INTERNALVERSION;
	 
	 // filelist
	 Utils::FileList("./data/languages", filelist);
	 if(int(filelist.size()) == 0){
		 Game::logfile("There's no language module available on \"./data/languages\"");
		 MsgBox("There's no language module available on \"./data/languages\"");
		 Game::abort();
	 }
	 
	 // Is there an English Language module available?
	 if(!exists("./data/languages/English.lgm")){
		 Game::logfile("Couldn't find file \"./data/languages/English.lgm\". You MUST have this file installed in order to run the game.");
		 MsgBox("Cannot find file \"./data/languages/English.lgm\". You MUST have this file installed in order to run the game.");
		 Game::abort();
	 }
	 
	 // Setting-up the languages
	 for(it = filelist.begin(); it != filelist.end(); ++it)
		 lang.push_back("./data/languages/" + (*it));
	 
	 // Checking the language modules' versions
	 for(itl = lang.begin(); itl != lang.end(); ++itl){
		 if(itl->pdata.ReadString("version") != language_sign){
			 Game::logfile("Incorrect version for language module \"%s\". It's \"%s\", when it should be \"%s\". Please update or delete it (on the \"data/languages/\" folder).",itl->language.c_str(), itl->pdata.ReadString("version").c_str(), language_sign.c_str());
			 MsgBox("Incorrect version for language module \"%s\". It's \"%s\", when it should be \"%s\". Please update or delete it (on the \"data/languages/\" folder).",itl->language.c_str(), itl->pdata.ReadString("version").c_str(), language_sign.c_str());
			 Game::abort();
		 }
	 }
	 
	 // And we're done!
	 if(int(filelist.size()) > 1) Game::logfile("All the language modules have been loaded. There are %d languages available.",int(filelist.size()));
	 else Game::logfile("The language module has been loaded. There's 1 language available.");	 
 }
 
 // loadFonts()
 void Data::loadFonts(){
	 int a;
	 for(a=0;a<int(sFont.size());a++){
		 font.push_back(alfont_load_font(sFont[a].c_str()));
		 if(! (*(font.end()-1)) ){
			 Game::logfile("Couldn't load font \"%s\"", sFont[a].c_str());
			 MsgBox("Couldn't load font \"%s\"", sFont[a].c_str());
			 Game::abort();
		 }
		 alfont_set_font_size((*(font.end()-1)), 20);
	 }
 }
 
 // loadCharacters()
 void Data::loadCharacters(){
	 vector<string> filelist;
	 vector<string>::iterator it;
	 
	 Utils::FileList("./data/characters", filelist, "*.mdf");
	 
	 for(it = filelist.begin(); it != filelist.end(); ++it)
		 character.push_back("./data/characters/" + (*it));
	 
	 if(filelist.size() <= 0){
		 Game::logfile("It's impossible to run the game without having any characters to play with. Please insert some in the \"data/characters\" folder... Or reinstall the game ;)");
		 MsgBox("It's impossible to run the game without having any characters to play with. Please insert some in the \"data/characters\" folder... Or reinstall the game ;)");
		 Game::abort();
	 }
 }
 
 // loadMapTiles()
 void Data::loadMapTiles(){
	 int a, animspeed;
	 vector<string> filelist;
	 CIni ini("./data/maptiles/tiles.mdf");
	 
	 Utils::FileList("./data/maptiles", filelist, "*_0.pcx");
	 if(filelist.size() <= 0){
		 Game::logfile("It's impossible to run the game without having a map tile. Please reinstall the game.");
		 MsgBox("It's impossible to run the game without having a map tile. Please reinstall the game.");
		 Game::abort();
	 }
	 if(ini.ReadInt("tilecount") != int(filelist.size())){
		 Game::logfile("mapTiles: tilecount (%d) variable doesn't match with the number of available tiles (%d). Please reinstall the game.", ini.ReadInt("tilecount") , int(filelist.size()));
		 MsgBox("mapTiles: tilecount (%d) variable doesn't match with the number of available tiles (%d). Please reinstall the game.", ini.ReadInt("tilecount") , int(filelist.size()));
		 Game::abort();
	 }
	 
	 for(a=0;a<int(filelist.size());a++){
		 mapTile.push_back(a);
		 animspeed = ini.ReadInt("tile(" + StringUtils::int2str(a) + ").animSpeed");
		 mapTile[a].animSpeed = (animspeed <= 0) ? 10 : animspeed;
	 }
	 
 }
 
 // loadAll()
 void Data::loadAll(){
	 
	 BITMAP *loading, *loading_bar;
	 int a = 0, percentage = 0;
	 string sLoadImage[] = { "./data/images/loading.pcx" , "./data/images/loading_bar.pcx" };
	 
	 // Pre-loader stuff
	 for(a=0;a<2;a++){
		 if(!exists(sLoadImage[a].c_str())){
			 Game::logfile("Couldn't load image \"%s\": file doesn't exist", sLoadImage[a].c_str());
			 MsgBox("Cannot load image \"%s\": file doesn't exist", sLoadImage[a].c_str());
			 Game::abort();
		 }
	 }
	 
	 loading = load_bitmap(sLoadImage[0].c_str(), NULL);
	 loading_bar = load_bitmap(sLoadImage[1].c_str(), NULL);
	 
	 if(!loading || !loading_bar){
		 Game::logfile("Loading screen: error while reading bitmaps... %s",allegro_error);
		 MsgBox("Loading screen: error while reading bitmaps... %s",allegro_error);
		 Game::abort();
	 }
	 
	 // Showing the "Loading" screen
	 
	 MGSLEEP(1000);
 #ifdef MGSLEEP_ENABLED
	 FadeFX::fadeIn(loading, 16);
 #endif
	 percentage = 0;
	 blit(loading, Game::bmpBuffer, 0, 0, 0, 0, loading->w, loading->h);
	 Game::render();
	 MGSLEEP(1000);
	 
	 // -------------------------------
	 
	 // Loading stuff...
	 ReadLists();
	 percentage = 5;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadLanguages();
	 percentage = 10;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadImages();
	 percentage = 20;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadAnimations();
	 percentage = 30;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadBgAnimations();
	 percentage = 40;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadEnemies();
	 percentage = 50;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadItems();
	 percentage = 60;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadSamples();
	 percentage = 70;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadMusics();
	 percentage = 80;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadFonts();
	 percentage = 90;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadCharacters();
	 percentage = 95;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 loadMapTiles();
	 percentage = 100;
	 blit(loading_bar, Game::bmpBuffer, 0, 0, 84, 155, int(percentage*1.5), loading_bar->h);
	 Game::render();
	 MGSLEEP(25);
	 
	 FadeFX::fadeOut(0, 32);
	 
	 destroy_bitmap(loading);
	 destroy_bitmap(loading_bar);
	 
 }
 
 /////////////////////////////////////////////////////////
 
 // releaseImages()
 void Data::releaseImages(){
	 vector<BITMAP*>::iterator it;
	 for(it = image.begin(); it != image.end(); ++it)
		 destroy_bitmap((*it));
 }
 
 // releaseAnimations()
 void Data::releaseAnimations(){
	 vector<CAnimation>::iterator it;
	 for(it = animation.begin(); it != animation.end(); ++it)
		 it->release();
 }
 
 // releaseBgAnimations()
 void Data::releaseBgAnimations(){
	 vector<CAnimation>::iterator it;
	 
	 for(it = superBgAnimation.begin(); it != superBgAnimation.end(); ++it)
		 it->release();
	 
	 for(it = bgAnimation.begin(); it != bgAnimation.end(); ++it)
		 it->release();
	 
	 for(it = fgAnimation.begin(); it != fgAnimation.end(); ++it)
		 it->release();
 }
 
 // releaseEnemies()
 void Data::releaseEnemies(){
	 vector<CEnemy>::iterator it;
	 for(it = enemy.begin(); it != enemy.end(); ++it)
		 it->release();
 }
 
 // releaseItems()
 void Data::releaseItems(){
	 vector<CItem>::iterator it;
	 for(it = item.begin(); it != item.end(); ++it)
		 it->release();
 }
 
 // releaseSamples()
 void Data::releaseSamples(){
	 vector<SAMPLE*>::iterator it;
	 for(it = sample.begin(); it != sample.end(); ++it)
		 destroy_sample((*it));
 }
 
 // releaseMusics()
 void Data::releaseMusics(){
	 vector<FMUSIC_MODULE*>::iterator it;
	 for(it = music.begin(); it != music.end(); ++it)
		 FMUSIC_FreeSong((*it));
 }
 
 // releaseFonts()
 void Data::releaseFonts(){
	 vector<ALFONT_FONT*>::iterator it;
	 for(it = font.begin(); it != font.end(); ++it)
		 alfont_destroy_font((*it));
 }
 
 // releaseCharacters()
 void Data::releaseCharacters(){
	 vector<CPlayer>::iterator it;
	 for(it = character.begin(); it != character.end(); ++it)
		 it->release();
 }
 
 // releaseMapTiles()
 void Data::releaseMapTiles(){
	 vector<CAnimation>::iterator it;
	 for(it = mapTile.begin(); it != mapTile.end(); ++it)
		 it->release();
 }
 
 // releaseAll()
 void Data::releaseAll(){
	 releaseImages();
	 releaseAnimations();
	 releaseBgAnimations();
	 releaseEnemies();
	 releaseItems();
	 releaseSamples();
	 releaseMusics();
	 releaseFonts();
	 releaseCharacters();
	 releaseMapTiles();
 }
 
 
 
 
 
 
 
 /////////////////// Other Methods //////////////////////////
 // CLanguage::ReadFile(): reads a language file
 void CLanguage::ReadFile(const string& sFile){
	 if(!pdata.ReadFile(sFile)){
		 Game::logfile("Couldn't load language module \"%s\"",sFile.c_str());
		 MsgBox("Couldn't load language module \"%s\"",sFile.c_str());
		 Game::abort();
	 }
	 
	 vector<string> lngv;
	 StringUtils::Split(sFile, "/", lngv);
	 language = StringUtils::Left(*(lngv.end()-1),(lngv.end()-1)->length()-4);
 }

 
 // Lang(x): returns a string of the current language
 string Data::Lang(string x){
	 if(Game::iLanguage<0 || Game::iLanguage>=int(lang.size()))
		 return "Unknown language";
		 
	 return lang[Game::iLanguage].pdata.ReadString(x);
 }
