/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include "Frames.h"
 #include "Map.h"

 // defines
 #define SELECTEDTHING_TILE				0
 #define SELECTEDTHING_ITEM				1
 #define SELECTEDTHING_ENEMY			2
 #define DELFLAG_ITEMS					1
 #define DELFLAG_ENEMIES				2
 
 // vars
 bool bLevedit_quit;						// quit?
 bool bLevedit_modified;					// map has been modified (after loaded / created) ?
 int iLevedit_selectedthing;				// tile, item, enemy?
 int iLevedit_selectedthingID;				// selected [tile, item, enemy] ID
 int iLevedit_currentLayer;					// current layer (1, 2, 3)?
 int iLevedit_scrollx;						// scroll x
 int iLevedit_scrolly;						// scroll y
 bool bLevedit_canInsertNewObjects;			// can insert new objects?
 string sLevedit_status;					// statusbar msg
 CTimer tmrLevedit_status(3500);			// statusbar timer
 vector<int> vMapTileInitialXOffset;		// initial X offset (for all tiles)
 vector<int> vMapTileInitialYOffset;		// initial Y offset (for all tiles)
 CMap LvMap;								// the level
 
 // vars from files
 int iLevedit_maxMapWidth;
 int iLevedit_maxMapHeight;
 
 // func prototypes
 void levedit_events();
 void levedit_showhelp(int page);
 void levedit_changestatus(string msg);
 void levedit_confirmexit();
 void levedit_drawmap();
 void levedit_newlevel();
 void levedit_loadlevel();
 void levedit_savelevel();
 void levedit_selector();
 void levedit_resizemap(int direction);
 void levedit_deleteAnyObjectAt(int mapxpos, int mapypos, char delflag);
 void levedit_changeoffset(int tilex, int tiley, int direction);
 void levedit_changeproperties();
 
 // levedit_setup_vMapTileInitialOffsets()
 void levedit_setup_vMapTileInitialOffsets(){
	 
	 CIni ini("./data/maptiles/tiles.mdf");
	 int a;
	 
	 for(a=0;a<int(Data::mapTile.size());a++){
		 vMapTileInitialXOffset.push_back(ini.ReadInt("tile(" + StringUtils::int2str(a) + ").initialXOffset"));
		 vMapTileInitialYOffset.push_back(ini.ReadInt("tile(" + StringUtils::int2str(a) + ").initialYOffset"));
	 }
	 
 }
 
 // --------------------------------------------
 // level_editor(): Level Editor
 // --------------------------------------------
 void Frame::level_editor(){
	 
	 bool bInputDeviceUseJoystick;
	 CIni ini("./data/smwx2.ini");
	 
	 iLevedit_maxMapWidth = ini.ReadInt("lvedit_maxmapwidth");
	 iLevedit_maxMapHeight = ini.ReadInt("lvedit_maxmapheight");
	 if(iLevedit_maxMapWidth < 10 && iLevedit_maxMapWidth != -1) iLevedit_maxMapWidth = 10;
	 if(iLevedit_maxMapHeight < 8 && iLevedit_maxMapHeight != -1) iLevedit_maxMapHeight = 8;
	 
	 bLevedit_quit = false;
	 bLevedit_modified = false;
	 iLevedit_selectedthing = SELECTEDTHING_TILE;
	 iLevedit_selectedthingID = 0;
	 iLevedit_currentLayer = 1;
	 iLevedit_scrollx = 0;
	 iLevedit_scrolly = 0;
	 bLevedit_canInsertNewObjects = true;
	 bInputDeviceUseJoystick = Player1_Control.bUseJoystick;
	 Player1_Control.bUseJoystick = false;
	 
	 levedit_setup_vMapTileInitialOffsets();
	 sLevedit_status = Data::Lang("levedit_ready");
	 levedit_showhelp(1);
	 LvMap.createnew(10,8);
	 
	 while(!bLevedit_quit){
		 while(Game::MayRun()){
			 
			 clear_bitmap(Game::bmpBuffer);
			 levedit_drawmap();
			 
			 // Events
			 levedit_events();
			 Game::events();
		 }
		 Game::render();
	 }
	 
	 Player1_Control.bUseJoystick = bInputDeviceUseJoystick;
	 
 }
 
 
 
 
 
 // levedit_events()
 void levedit_events(){
	 
	 static int x, y, i, j, w, h, cursor_x, cursor_y, lvedit_mousex, lvedit_mousey, cl[2] = { makecol(255,255,0) , makecol(255,255,255) };
	 static string statusbar;
	 static ALFONT_FONT **fnt = &Data::font[_ARIAL6];
	 static CEvery tmr(5);
	 
	 // Grid
	 vline(Game::bmpBuffer, BUFFER_W - 1, 0, BUFFER_H - 16, cl[0]);
	 for(x=0;x<10;x++) vline(Game::bmpBuffer, x*32, 0, BUFFER_H - 16, cl[0]);
	 for(y=0;y<8;y++) hline(Game::bmpBuffer, 0, y*32, BUFFER_W, cl[0]);
	 
	 // Status bar
	 if(tmrLevedit_status.MayRun())
		 sLevedit_status = Data::Lang("levedit_ready");
	 
	 statusbar = Data::Lang("levedit_sbhelp") + " | " +
				 Data::Lang("levedit_sbsize") + " " + StringUtils::int2str(LvMap.width()) + " x " + StringUtils::int2str(LvMap.height()) + " | " +
				 Data::Lang("levedit_sbpos") + " " + StringUtils::int2str(lvedit_mousex+1) + " , " + StringUtils::int2str(lvedit_mousey+1) + " | " +
				 sLevedit_status;
				 
	 alfont_textprintf_color(Game::bmpBuffer, *fnt, 2, BUFFER_H - 2 - alfont_text_height(*fnt), -1, 0, statusbar.c_str());
	 
	 // Selected thing
	 cursor_x = mouse_x - (mouse_x % 32);
	 cursor_y = mouse_y - (mouse_y % 32);
	 if(int(mouse_y / 32) < 7){
		 switch(iLevedit_selectedthing){
		 case SELECTEDTHING_TILE:
			 w = Data::mapTile[iLevedit_selectedthingID].getImage(0)->w;
			 h = Data::mapTile[iLevedit_selectedthingID].getImage(0)->h;
			 i = j = 0;
			 if(w < 32) i = 16 - (w/2);
			 if(h < 32) j = 16 - (h/2);
			 masked_blit(Data::mapTile[iLevedit_selectedthingID].getImage(0), Game::bmpBuffer, 0, 0, cursor_x+i, cursor_y+j, 32, 32);
			 break;
			 
		 case SELECTEDTHING_ITEM:
			 w = Data::item[iLevedit_selectedthingID].getImage(0)->w;
			 h = Data::item[iLevedit_selectedthingID].getImage(0)->h;
			 i = j = 0;
			 if(w < 32) i = 16 - (w/2);
			 if(h < 32) j = 16 - (h/2);
			 masked_blit(Data::item[iLevedit_selectedthingID].getImage(0), Game::bmpBuffer, 0, 0, cursor_x+i, cursor_y+j, 32, 32);
			 break;
			 
		 case SELECTEDTHING_ENEMY:
			 w = Data::enemy[iLevedit_selectedthingID].getImage(0)->w;
			 h = Data::enemy[iLevedit_selectedthingID].getImage(0)->h;
			 i = j = 0;
			 if(w < 32) i = 16 - (w/2);
			 if(h < 32) j = 16 - (h/2);
			 masked_blit(Data::enemy[iLevedit_selectedthingID].getImage(0), Game::bmpBuffer, 0, 0, cursor_x+i, cursor_y+j, 32, 32);
			 break;
		 }
	 }
	 
	 // Mouse
	 if(int(mouse_y / 32) < 7){
		 rect(Game::bmpBuffer, mouse_x - (mouse_x % 32), mouse_y - (mouse_y % 32), (mouse_x - (mouse_x % 32)) + 32, (mouse_y - (mouse_y % 32)) + 32, cl[1]);
		 if(int(mouse_x / 32) >= 9) vline(Game::bmpBuffer, (mouse_x - (mouse_x % 32)) + 31, mouse_y - (mouse_y % 32), (mouse_y - (mouse_y % 32)) + 32, cl[1]);
		 
		 if(mouse_y >= 0 && mouse_x >= 0 && mouse_x < BUFFER_W){
			 lvedit_mousex = int(mouse_x/32) + iLevedit_scrollx;
			 lvedit_mousey = int(mouse_y/32) + iLevedit_scrolly;
		 }
	 }
	 draw_sprite(Game::bmpBuffer, mouse_sprite, mouse_x, mouse_y);
	 
	 // -- Insert new objects --
	 // is it locked?
	 if(!(mouse_b & 1))
		 bLevedit_canInsertNewObjects = true;
	 
	 // it's unlocked :)
	 if(bLevedit_canInsertNewObjects && (mouse_b & 1)){
		 bLevedit_modified = true;
		 switch(iLevedit_selectedthing){
		 // new tile
		 case SELECTEDTHING_TILE:
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->tile[iLevedit_currentLayer-1] = iLevedit_selectedthingID;
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->offset_x = vMapTileInitialXOffset[iLevedit_selectedthingID];
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->offset_y = vMapTileInitialYOffset[iLevedit_selectedthingID];
			 break;
			 
		 // new item
		 case SELECTEDTHING_ITEM:
			 levedit_deleteAnyObjectAt(lvedit_mousex, lvedit_mousey, DELFLAG_ITEMS | DELFLAG_ENEMIES);
			 LvMap.itemData.push_back(&Data::item[iLevedit_selectedthingID]);
			 (LvMap.itemData.end()-1)->alterableValue[5] = iLevedit_selectedthingID;
			 (LvMap.itemData.end()-1)->x = (lvedit_mousex*32) + (16 - ((LvMap.itemData.end()-1)->width()/2));
			 (LvMap.itemData.end()-1)->y = (lvedit_mousey*32) + (16 - ((LvMap.itemData.end()-1)->height()/2));
			 (LvMap.itemData.end()-1)->interval[0] = Data::item[iLevedit_selectedthingID].animInterval[0][IANIM_NORMAL];
			 (LvMap.itemData.end()-1)->interval[1] = Data::item[iLevedit_selectedthingID].animInterval[1][IANIM_NORMAL];
			 break;
			 
		 // new enemy
		 case SELECTEDTHING_ENEMY:
			 levedit_deleteAnyObjectAt(lvedit_mousex, lvedit_mousey, DELFLAG_ITEMS | DELFLAG_ENEMIES);
			 LvMap.enemyData.push_back(&Data::enemy[iLevedit_selectedthingID]);
			 (LvMap.enemyData.end()-1)->alterableValue[5] = iLevedit_selectedthingID;
			 (LvMap.enemyData.end()-1)->x = (lvedit_mousex*32) + (16 - ((LvMap.enemyData.end()-1)->width()/2));
			 (LvMap.enemyData.end()-1)->y = (lvedit_mousey*32) + (32 - (LvMap.enemyData.end()-1)->height()) + 1;
			 (LvMap.enemyData.end()-1)->interval[0] = Data::enemy[iLevedit_selectedthingID].animInterval[0][EANIM_WALK];
			 (LvMap.enemyData.end()-1)->interval[1] = Data::enemy[iLevedit_selectedthingID].animInterval[1][EANIM_WALK];
			 break;
		 }
	 }
	 
	 // -- Deleting existing stuff --
	 if(mouse_b & 2){
		 bLevedit_modified = true;
		 switch(iLevedit_selectedthing){
		 // deleting current tile
		 case SELECTEDTHING_TILE:
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->tile[iLevedit_currentLayer-1] = 0;
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->offset_x = 0;
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->offset_y = 0;
			 LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = 0;
			 break;
			 
		 // deleting an item
		 case SELECTEDTHING_ITEM:
			 levedit_deleteAnyObjectAt(lvedit_mousex, lvedit_mousey, DELFLAG_ITEMS);
			 break;
			 
		 // deleting an enemy
		 case SELECTEDTHING_ENEMY:
			 levedit_deleteAnyObjectAt(lvedit_mousex, lvedit_mousey, DELFLAG_ENEMIES);
			 break;
		 }
	 }
	 
	 // Hotkeys
	 if(key[KEY_ESC]) levedit_confirmexit();
	 if(key[KEY_F1]) levedit_showhelp(1);
	 if(key[KEY_F2]) levedit_showhelp(2);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_N]) levedit_newlevel();
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_S]) levedit_savelevel();
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_L]) levedit_loadlevel();
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_P]){ bLevedit_modified = true; levedit_changeproperties(); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_T]){ iLevedit_selectedthing = SELECTEDTHING_TILE; levedit_selector(); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_I]){ iLevedit_selectedthing = SELECTEDTHING_ITEM; levedit_selector(); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_E]){ iLevedit_selectedthing = SELECTEDTHING_ENEMY; levedit_selector(); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && !(key[KEY_LSHIFT] || key[KEY_RSHIFT]) && Player1_Control.keypress(K_UP)) levedit_resizemap(UP);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && !(key[KEY_LSHIFT] || key[KEY_RSHIFT]) && Player1_Control.keypress(K_DOWN)) levedit_resizemap(DOWN);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && !(key[KEY_LSHIFT] || key[KEY_RSHIFT]) && Player1_Control.keypress(K_LEFT)) levedit_resizemap(LEFT);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && !(key[KEY_LSHIFT] || key[KEY_RSHIFT]) && Player1_Control.keypress(K_RIGHT)) levedit_resizemap(RIGHT);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && (key[KEY_LSHIFT] || key[KEY_RSHIFT]) && key[KEY_UP]) levedit_changeoffset(lvedit_mousex, lvedit_mousey, UP);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && (key[KEY_LSHIFT] || key[KEY_RSHIFT]) && key[KEY_DOWN]) levedit_changeoffset(lvedit_mousex, lvedit_mousey, DOWN);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && (key[KEY_LSHIFT] || key[KEY_RSHIFT]) && key[KEY_LEFT]) levedit_changeoffset(lvedit_mousex, lvedit_mousey, LEFT);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && (key[KEY_LSHIFT] || key[KEY_RSHIFT]) && key[KEY_RIGHT]) levedit_changeoffset(lvedit_mousex, lvedit_mousey, RIGHT);
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_1]){ iLevedit_currentLayer = 1; levedit_changestatus(Data::Lang("levedit_currlayer")+StringUtils::int2str(iLevedit_currentLayer)); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_2]){ iLevedit_currentLayer = 2; levedit_changestatus(Data::Lang("levedit_currlayer")+StringUtils::int2str(iLevedit_currentLayer)); }
	 if((key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_3]){ iLevedit_currentLayer = 3; levedit_changestatus(Data::Lang("levedit_currlayer")+StringUtils::int2str(iLevedit_currentLayer)); }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_S]){ LvMap.start_x = lvedit_mousex; LvMap.start_y = lvedit_mousey; bLevedit_modified = true; }
	 
	 // masks
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_1]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_2]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_FULL; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_3]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_DEATH; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_4]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_5]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_6]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_7]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_8]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_9]){ bLevedit_modified = true; LvMap.tileData.pGet(lvedit_mousex, lvedit_mousey)->mask = TMASK_NONE; }
	 
	 // scrolling
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_UP] && tmr.MayRun()) iLevedit_scrolly--;
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_DOWN] && tmr.MayRun()) iLevedit_scrolly++;
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_LEFT] && tmr.MayRun()) iLevedit_scrollx--;
	 if(!(key[KEY_LCONTROL] || key[KEY_RCONTROL]) && key[KEY_RIGHT] && tmr.MayRun()) iLevedit_scrollx++;
	 if(iLevedit_scrolly < 0) iLevedit_scrolly = 0;
	 if(iLevedit_scrolly > LvMap.height() - 7) iLevedit_scrolly = LvMap.height() - 7; // 7, not 8. One tile is not drawed (vertical - the last tile)
	 if(iLevedit_scrollx < 0) iLevedit_scrollx = 0;
	 if(iLevedit_scrollx > LvMap.width() - 10) iLevedit_scrollx = LvMap.width() - 10;
	 
 }
 
 // levedit_showhelp(page)
 void levedit_showhelp(int page){
	 
	 ALFONT_FONT **fnt = &Data::font[_ARIAL6];
	 alfont_set_font_size(*fnt, 12);
	 
	 blit(Data::image[11], Game::bmpBuffer, 0, 0, 0, 0, Data::image[11]->w, Data::image[11]->h);
	 rect_trans(Game::bmpBuffer, 5, 5, 200, 230, 0, 85);
	 alfont_textprintf_color(Game::bmpBuffer, *fnt, 10, 7, -1, 0, Data::Lang("levedit_help" + StringUtils::int2str(page)).c_str());
	 
	 GSleep(500);
	 SuspendForKey();
	 play_sample(Data::sample[3]);
	 
 }
 
 // levedit_changestatus(msg)
 void levedit_changestatus(string msg){
	 
	 sLevedit_status = msg;
	 tmrLevedit_status.Reset();
	 
 }
 
 // levedit_confirmexit()
 void levedit_confirmexit(){
	 
	 play_sample(Data::sample[5]);
	 switch(alert3(Data::Lang("quit_msg0").c_str(), NULL, NULL, Data::Lang("yes").c_str(), Data::Lang("no").c_str(), Data::Lang("levedit_helpme").c_str(), 0, 0, 0)){
	 case 1:
		 bLevedit_quit = true;
		 break;
	case 3:
		 levedit_showhelp(1);
		 break;
	 }
	 
	 if(bLevedit_quit && bLevedit_modified){
		 play_sample(Data::sample[5]);
		 if(alert(Data::Lang("levedit_nmodf1").c_str(), Data::Lang("levedit_nmodf3").c_str(), NULL, Data::Lang("yes").c_str(), Data::Lang("no").c_str(), 0, 0) == 1)
			 levedit_savelevel();
	 }
	 
	 Timer::fix_speedcounter();
	 
 }
 
 // levedit_newlevel()
 void levedit_newlevel(){
	 
	 if(bLevedit_modified){
		 play_sample(Data::sample[5]);
		 if(alert(Data::Lang("levedit_nmodf1").c_str(), Data::Lang("levedit_nmodf2").c_str(), NULL, Data::Lang("yes").c_str(), Data::Lang("no").c_str(), 0, 0) == 1)
			 levedit_savelevel();
	 }
	 
	 LvMap.createnew(10,8);
	 play_sample(Data::sample[4]);
	 alert(Data::Lang("levedit_new").c_str(), NULL, NULL, "OK", NULL, 0, 0);
	 levedit_changestatus(Data::Lang("levedit_new"));
	 bLevedit_modified = false;
	 Timer::fix_speedcounter();
	 
 }
 
 // levedit_savelevel()
 void levedit_savelevel(){
	 
	 char dlgPath[6145]; // 6KB + 1 byte
	 bool bProtected;
	 
	 while(1){
		 strcpy(dlgPath, "./data/maps/");
		 if(file_select_ex(Data::Lang("levedit_save").c_str(), dlgPath, "smm", 1024, 250, 220) != 0){
			 if(exists(dlgPath)){ // Level already exists. Overwrite?
				 play_sample(Data::sample[5]);
				 if(alert(Data::Lang("levedit_saveerr1").c_str(), Data::Lang("levedit_saveerr2").c_str(), NULL, Data::Lang("yes").c_str(), Data::Lang("no").c_str(), 0, 0) == 2)
					 continue;
			 }
			 play_sample(Data::sample[5]); // Save protected?
			 bProtected = (alert(Data::Lang("levedit_savep1").c_str(), Data::Lang("levedit_savep2").c_str(), Data::Lang("levedit_savep3").c_str(), Data::Lang("yes").c_str(), Data::Lang("no").c_str(), 0, 0) == 1);
			 if(!LvMap.save(dlgPath, bProtected) || strcmp(dlgPath,"") == 0){ // Error?
				 play_sample(Data::sample[6]);
				 alert(Data::Lang("levedit_saveerr3").c_str(), NULL, NULL, "OK", NULL, 0, 0);
				 continue;
			 }
		 }else{
			 // Aborted?
			 levedit_changestatus(Data::Lang("levedit_aborted"));
			 break;
		 }
		 
		 // Operation succeeded! :)
		 play_sample(Data::sample[4]);
		 alert(Data::Lang("levedit_saved").c_str(), NULL, NULL, "OK", NULL, 0, 0);
		 levedit_changestatus(Data::Lang("levedit_saved"));
		 bLevedit_modified = false;
		 break;
	 }
	 
	 Timer::fix_speedcounter();
	 
 }
 
 // levedit_loadlevel()
 void levedit_loadlevel(){
	 
	 char dlgPath[6145]; // 6KB + 1 byte
	 
	 if(bLevedit_modified){
		 play_sample(Data::sample[5]);
		 if(alert(Data::Lang("levedit_nmodf1").c_str(), Data::Lang("levedit_nmodf2").c_str(), NULL, Data::Lang("yes").c_str(), Data::Lang("no").c_str(), 0, 0) == 1)
			 levedit_savelevel();
	 }
	 
	 strcpy(dlgPath, "./data/maps/");
	 if(file_select_ex(Data::Lang("levedit_load").c_str(), dlgPath, "smm", 1024, 250, 220) != 0){
		 if(!exists(dlgPath) || strcmp(dlgPath,"") == 0){ // Error - file doesn't exist
			 play_sample(Data::sample[6]);
			 alert(Data::Lang("levedit_loaderr1").c_str(), NULL, NULL, "OK", NULL, 0, 0);
			 levedit_changestatus(Data::Lang("levedit_aborted"));
			 Timer::fix_speedcounter();
			 return;
		 }
		 if(!LvMap.load(dlgPath, false)){ // Error - couldn't read map
			 play_sample(Data::sample[6]);
			 alert(Data::Lang("levedit_loaderr2").c_str(), Data::Lang("levedit_loaderr3").c_str(), Data::Lang("logfile").c_str(), "OK", NULL, 0, 0);
			 levedit_changestatus(Data::Lang("levedit_aborted"));
			 Timer::fix_speedcounter();
			 return;
		 }
	 }
	 
	 levedit_changestatus(Data::Lang("levedit_loaded"));
	 bLevedit_modified = false;
	 
 }
 
 // levedit_drawmap()
 void levedit_drawmap(){
	 
	 static int a, x, y, tx, ty, wht = makecol(0xFF, 0xFF, 0xFF);
	 
	 // tiles
	 for(y=0;y<7;y++){
		 for(x=0;x<10;x++){
			 tx = iLevedit_scrollx + x;
			 ty = iLevedit_scrolly + y;
			 for(a=0;a<3;a++){
				 if(LvMap.tileData.pGet(tx,ty)->tile[a] != 0){
					 // drawing the tile
					 draw_sprite(
						 Game::bmpBuffer ,
						 Data::mapTile[ LvMap.tileData.pGet(tx,ty)->tile[a] ].getImage(0) ,
						 x*32 + LvMap.tileData.pGet(tx,ty)->offset_x ,
						 y*32 + LvMap.tileData.pGet(tx,ty)->offset_y
					 );
				 }
			 }
		 }
	 }
	 
	 // items
	 for(a=0;a<int(LvMap.itemData.size());a++){
		 LvMap.itemData[a].RunAnimation();
		 if(LvMap.itemData[a].x >= iLevedit_scrollx*32 && LvMap.itemData[a].x < (iLevedit_scrollx+10)*32 && LvMap.itemData[a].y >= iLevedit_scrolly*32 && LvMap.itemData[a].y < (iLevedit_scrolly+7)*32){
			 LvMap.itemData[a].x -= iLevedit_scrollx*32;
			 LvMap.itemData[a].y -= iLevedit_scrolly*32;
			 LvMap.itemData[a].draw(Game::bmpBuffer);
			 LvMap.itemData[a].x += iLevedit_scrollx*32;
			 LvMap.itemData[a].y += iLevedit_scrolly*32;
		 }
	 }
	 
	 // enemies
	 for(a=0;a<int(LvMap.enemyData.size());a++){
		 LvMap.enemyData[a].RunAnimation();
		 if(LvMap.enemyData[a].x >= iLevedit_scrollx*32 && LvMap.enemyData[a].x < (iLevedit_scrollx+10)*32 && LvMap.enemyData[a].y >= iLevedit_scrolly*32 && LvMap.enemyData[a].y < (iLevedit_scrolly+7)*32){
			 LvMap.enemyData[a].x -= iLevedit_scrollx*32;
			 LvMap.enemyData[a].y -= iLevedit_scrolly*32;
			 LvMap.enemyData[a].draw(Game::bmpBuffer);
			 LvMap.enemyData[a].x += iLevedit_scrollx*32;
			 LvMap.enemyData[a].y += iLevedit_scrolly*32;
		 }
	 }
	 
	 // drawing the masks
	 for(y=0;y<7;y++){
		 for(x=0;x<10;x++){
			 tx = iLevedit_scrollx + x;
			 ty = iLevedit_scrolly + y;
			 switch(LvMap.tileData.pGet(tx,ty)->mask){
			 case TMASK_FULL:
				 line(Game::bmpBuffer, x*32, y*32, x*32+32, y*32+32, wht);
				 line(Game::bmpBuffer, x*32+32, y*32, x*32, y*32+32, wht);
				 break;
				 
			 case TMASK_DEATH:
				 text_mode(0);
				 textout_centre(Game::bmpBuffer, font, "D", x*32+16, y*32+12, wht);
				 break;
			 }
		 }
	 }
	 
	 // initial player position
	 if(LvMap.start_x >= iLevedit_scrollx && LvMap.start_x < iLevedit_scrollx + 10){
		 if(LvMap.start_y >= iLevedit_scrolly && LvMap.start_y < iLevedit_scrolly + 7){
			 text_mode(0);
			 textout_centre(Game::bmpBuffer, font, "S", (LvMap.start_x-iLevedit_scrollx)*32+16, (LvMap.start_y-iLevedit_scrolly)*32+12, wht);
		 }
	 }
	 
 }
 
 // levedit_selector()
 void levedit_selector(){
	 
	 int a, x, y, i, j, w, h, size = 0, scrolly = 0, lines = 0, cl[2] = { makecol(255,255,255) , makecol(0,204,255) };
	 bool bQuitSelector;
	 string statusbar;
	 ALFONT_FONT **fnt = &Data::font[_ARIAL6];
	 BITMAP *bg;
	 float bg_scrolly;
	 
	 // Setup
	 bQuitSelector = false;
	 scrolly = 0;
	 bLevedit_canInsertNewObjects = false;
	 alfont_set_font_size(*fnt, 12);
	 
	 switch(iLevedit_selectedthing){
	 case SELECTEDTHING_TILE: statusbar = Data::Lang("levedit_tselect") + " - "; lines = int(Data::mapTile.size())/32; break;
	 case SELECTEDTHING_ITEM: statusbar = Data::Lang("levedit_iselect") + " - "; lines = int(Data::item.size())/32; break;
	 case SELECTEDTHING_ENEMY: statusbar = Data::Lang("levedit_eselect") + " - "; lines = int(Data::enemy.size())/32; break;
	 }
	 statusbar += Data::Lang("levedit_scroll");
	 
	 bg_scrolly = 0.0f;
	 bg = create_bitmap(Data::image[8]->w, Data::image[8]->h);
	 blit(Data::image[8], bg, 0, 0, 0, 0, bg->w, bg->h);
	 rect_trans(bg, 0, 0, bg->w, bg->h, 0, 100);
	 
	 // click
	 play_sample(Data::sample[3]);
	 
	 // Main loop	 
	 while(!bQuitSelector){
		 while(Game::MayRun()){
			 
			 // checking size
			 switch(iLevedit_selectedthing){
			 case SELECTEDTHING_TILE: size = int(Data::mapTile.size()); break;
			 case SELECTEDTHING_ITEM: size = int(Data::item.size()); break;
			 case SELECTEDTHING_ENEMY: size = int(Data::enemy.size()); break;
			 }
			 
			 // drawing bg
			 bg_scrolly -= 0.25f;
			 if(bg_scrolly <= -BUFFER_H) bg_scrolly = 0.0f;
			 blit(bg, Game::bmpBuffer, 0, 0, 0, int(bg_scrolly), bg->w, bg->h);
			 blit(bg, Game::bmpBuffer, 0, 0, 0, int(bg_scrolly)+BUFFER_H, bg->w, bg->h);
			 
			 // Drawing stuff
			 x = 0;
			 y = scrolly * 32;
			 for(a=0;a<size;a++){
				 switch(iLevedit_selectedthing){
				 case SELECTEDTHING_TILE:
					 w = Data::mapTile[a].getImage(0)->w;
					 h = Data::mapTile[a].getImage(0)->h;
					 i = j = 0;
					 if(w < 32) i = 16 - (w/2);
					 if(h < 32) j = 16 - (h/2);
					 masked_blit(Data::mapTile[a].getImage(0), Game::bmpBuffer, 0, 0, x+i, y+j, 32, 32);
					 break;
					 
				 case SELECTEDTHING_ITEM:
					 w = Data::item[a].getImage(0)->w;
					 h = Data::item[a].getImage(0)->h;
					 i = j = 0;
					 if(w < 32) i = 16 - (w/2);
					 if(h < 32) j = 16 - (h/2);
					 masked_blit(Data::item[a].getImage(0), Game::bmpBuffer, 0, 0, x+i, y+j, 32, 32);
					 break;
					 
				 case SELECTEDTHING_ENEMY:
					 w = Data::enemy[a].getImage(0)->w;
					 h = Data::enemy[a].getImage(0)->h;
					 i = j = 0;
					 if(w < 32) i = 16 - (w/2);
					 if(h < 32) j = 16 - (h/2);
					 masked_blit(Data::enemy[a].getImage(0), Game::bmpBuffer, 0, 0, x+i, y+j, 32, 32);
					 break;
				 }
				 
				 x += 32;
				 if(x >= 320){
					 x = 0;
					 y += 32;
				 }
				 
				 if(y >= 224) break;
			 }
			 
			 // Grid
			 vline(Game::bmpBuffer, BUFFER_W - 1, 0, BUFFER_H - 16, cl[0]);
			 for(x=0;x<10;x++) vline(Game::bmpBuffer, x*32, 0, BUFFER_H - 16, cl[0]);
			 for(y=0;y<8;y++) hline(Game::bmpBuffer, 0, y*32, BUFFER_W, cl[0]);
			 
			 // Status bar
			 alfont_textprintf_color(Game::bmpBuffer, *fnt, 2, BUFFER_H - 2 - alfont_text_height(*fnt), -1, 0, statusbar.c_str());
			 
			 // Mouse
			 if(int(mouse_y / 32) < 7){
				 rect(Game::bmpBuffer, mouse_x - (mouse_x % 32), mouse_y - (mouse_y % 32), (mouse_x - (mouse_x % 32)) + 32, (mouse_y - (mouse_y % 32)) + 32, cl[1]);
				 if(int(mouse_x / 32) >= 9) vline(Game::bmpBuffer, (mouse_x - (mouse_x % 32)) + 31, mouse_y - (mouse_y % 32), (mouse_y - (mouse_y % 32)) + 32, cl[1]);
			 }
			 draw_sprite(Game::bmpBuffer, mouse_sprite, mouse_x, mouse_y);
			 
			 // Scroll Y
			 if(Player1_Control.keypress(K_UP) && scrolly > (-lines) + 7){ play_sample(Data::sample[3]); scrolly--; }
			 if(Player1_Control.keypress(K_DOWN) && scrolly > 0){ play_sample(Data::sample[3]); scrolly++; }
			 
			 // I'm out!
			 if(mouse_x > 0 && mouse_x < BUFFER_W && int(mouse_y / 32) < 7 && (mouse_b & 1)){
				 iLevedit_selectedthingID = (int(mouse_y/32)*10) + int(mouse_x/32) - (scrolly*10);
				 if(iLevedit_selectedthingID < 0 || iLevedit_selectedthingID >= size) iLevedit_selectedthingID = 0;
				 bQuitSelector = true;
			 }
			 
			 // Events
			 Game::events();
		 }
		 Game::render();
	 }
	 
	 // And we're done!
	 destroy_bitmap(bg);
	 play_sample(Data::sample[3]);
	 
 }
 
 // _levedit_resizemap_checkforstuff()
 // :: deletes any item & enemy outside the map
 void _levedit_resizemap_checkforstuff(){
	 
	 int a;
	 
	 for(a=0;a<int(LvMap.itemData.size());a++){
		 if(LvMap.itemData[a].x >= LvMap.width()*32 || LvMap.itemData[a].y >= LvMap.height()*32)
			 LvMap.itemData.erase( (LvMap.itemData.begin()+a) );
	 }
	 
	 for(a=0;a<int(LvMap.enemyData.size());a++){
		 if(LvMap.enemyData[a].x >= LvMap.width()*32 || LvMap.enemyData[a].y >= LvMap.height()*32)
			 LvMap.enemyData.erase( (LvMap.enemyData.begin()+a) );
	 }
	 
 }
 
 // levedit_resizemap(direction)
 void levedit_resizemap(int direction){
	 
	 int w, h;
	 w = LvMap.width();
	 h = LvMap.height();
	 bLevedit_modified = true;
	 
	 switch(direction){
	 case UP:
		 if(h > 8){
			 LvMap.tileData.resize(w, h-1);
			 play_sample(Data::sample[3]);
		 }
		 break;
		 
	 case DOWN:
		 if(iLevedit_maxMapHeight != -1 && h+1 > iLevedit_maxMapHeight){
			 play_sample(Data::sample[5]);
			 alert(Data::Lang("levedit_sizeerr1").c_str(), Data::Lang("levedit_sizeerr2b").c_str(), Data::Lang("levedit_sizeerr3").c_str(), "OK", NULL, 0, 0);
			 play_sample(Data::sample[5]);
			 alert(Data::Lang("levedit_sizeerr4").c_str(), Data::Lang("levedit_sizeerr5").c_str(), NULL, "OK", NULL, 0, 0);
			 Timer::fix_speedcounter();
			 break;
		 }
		 LvMap.tileData.resize(w, h+1);
		 play_sample(Data::sample[3]);
		 break;
		 
	 case LEFT:
		 if(w > 10){
			 LvMap.tileData.resize(w-1, h);
			 play_sample(Data::sample[3]);
		 }
		 break;
		 
	 case RIGHT:
		 if(iLevedit_maxMapWidth != -1 && w+1 > iLevedit_maxMapWidth){
			 play_sample(Data::sample[5]);
			 alert(Data::Lang("levedit_sizeerr1").c_str(), Data::Lang("levedit_sizeerr2a").c_str(), Data::Lang("levedit_sizeerr3").c_str(), "OK", NULL, 0, 0);
			 play_sample(Data::sample[5]);
			 alert(Data::Lang("levedit_sizeerr4").c_str(), Data::Lang("levedit_sizeerr5").c_str(), NULL, "OK", NULL, 0, 0);
			 Timer::fix_speedcounter();
			 break;
		 }
		 LvMap.tileData.resize(w+1, h);
		 play_sample(Data::sample[3]);
		 break;
	 }
	 
	 _levedit_resizemap_checkforstuff();
	 
 }
 
 // levedit_deleteAnyObjectAt(mapxpos, mapypos)
 void levedit_deleteAnyObjectAt(int mapxpos, int mapypos, char delflag){
	 
	 int a;
	 
	 // seeking for items...
	 if(delflag & DELFLAG_ITEMS){
		 for(a=0;a<int(LvMap.itemData.size());a++){
			 if(int(LvMap.itemData[a].x/32) == mapxpos){
				 if(int(LvMap.itemData[a].y/32) == mapypos){
					 LvMap.itemData.erase(LvMap.itemData.begin()+a);
					 return; // only one object is allowed per tile,
							 // so we can return; to save time
				 }
			 }
		 }
	 }
	 
	 // seeking for enemies...
	 if(delflag & DELFLAG_ENEMIES){
		 for(a=0;a<int(LvMap.enemyData.size());a++){
			 if(int(LvMap.enemyData[a].x/32) == mapxpos){
				 if(int(LvMap.enemyData[a].y/32) == mapypos){
					 LvMap.enemyData.erase(LvMap.enemyData.begin()+a);
					 return;
				 }
			 }
		 }
	 }
	 
	 // nothing was found.
 }
 
 // levedit_changeoffset( tile_xpos, tile_ypos, direction)
 void levedit_changeoffset(int tilex, int tiley, int direction){
	 
	 CMappos *tile = LvMap.tileData.pGet(tilex, tiley);
	 bLevedit_modified = true;
	 
	 switch(direction){
	 case UP: tile->offset_y--; break;
	 case DOWN: tile->offset_y++; break;
	 case LEFT: tile->offset_x--; break;
	 case RIGHT: tile->offset_x++; break;
	 }
	 
	 if(tile->offset_x < -16) tile->offset_x = 16;
	 if(tile->offset_x > 16) tile->offset_x = -16;	 
	 if(tile->offset_y < -16) tile->offset_y = 16;
	 if(tile->offset_y > 16) tile->offset_y = -16;
	 
 }

 
 // =====================================================================
 // =====================================================================
 // =====================================================================
 
 // Level properties (GUI stuff)
 char dlg_stagename[181] = "";
 int dlg_winconditions, dlg_music, dlg_superbg, dlg_bg, dlg_fg;
 char dlg_strtable[20][100];
 char **d_listbox_idmusic_strs;
 char **d_listbox_idsuperbg_strs;
 char **d_listbox_idbg_strs;
 char **d_listbox_idfg_strs;
 BITMAP *dlg_picview;
 extern DIALOG dlg[];
 
 // GUI-related functions
 char *d_listbox_idmusic(int index, int *list_size){
	 if(index < 0){
		 *list_size = int( Data::music.size() ); //sizeof(d_listbox_idmusic_strs) / sizeof(char*);
		 return NULL;
	 }else{
		 return d_listbox_idmusic_strs[index];
	 }
 }

 char *d_listbox_idsuperbg(int index, int *list_size){
	 if(index < 0){
		 *list_size = int( Data::superBgAnimation.size() ); //sizeof(d_listbox_idsuperbg_strs) / sizeof(char*);
		 return NULL;
	 }else{
		 return d_listbox_idsuperbg_strs[index];
	 }
 }

 char *d_listbox_idbg(int index, int *list_size){
	 if(index < 0){
		 *list_size = int( Data::bgAnimation.size() ); //sizeof(d_listbox_idbg_strs) / sizeof(char*);
		 return NULL;
	 }else{
		 return d_listbox_idbg_strs[index];
	 }
 }

 char *d_listbox_idfg(int index, int *list_size){
	 if(index < 0){
		 *list_size = int( Data::fgAnimation.size() ); //sizeof(d_listbox_idfg_strs) / sizeof(char*);
		 return NULL;
	 }else{
		 return d_listbox_idfg_strs[index];
	 }
 }

 int d_playmusic(DIALOG *d){
	 char buf[100] = "";
	 
	 dlg_music = dlg[4].d1;
	 FMUSIC_PlaySong(Data::music[dlg_music]);
	 sprintf(buf, "%s [%03d]...", dlg_strtable[12], dlg_music);
	 alert(buf, "", "", "OK", NULL, 0, 0);
	 FMUSIC_StopSong(Data::music[dlg_music]);
	 
	 return D_O_K;
 }

 int d_picviewer(DIALOG *d){
	 BITMAP *temp = NULL;
 	 //char buf[100] = "";
 	 
	 switch(d->d1){
 	 case 0: temp = Data::superBgAnimation[ dlg[6].d1 ].getImage(0); break;
 	 case 1: temp = Data::bgAnimation[ dlg[8].d1 ].getImage(0); break;
 	 case 2: temp = Data::fgAnimation[ dlg[10].d1 ].getImage(0); break;
 	 }
 	 
 	 clear_to_color(dlg_picview, makecol(50, 130, 205));
	 masked_stretch_blit(temp, dlg_picview, 0, 0, temp->w, temp->h, 0, 0, dlg_picview->w, dlg_picview->h);
	 show_mouse(NULL);
	 object_message(&dlg[17], MSG_DRAW, 0);
	 show_mouse(screen);
 	 //sprintf(buf, "%s [%03d]", dlg_strtable[13], dlg[target].d1);
	 //play_sample(Data::sample[5]);
 	 //alert(buf, "", "", "OK", NULL, 0, 0);
 	 
	 return D_O_K;
 }
 
 // Level Editor functions
 // functions
 void levedit_d_loadall(){
	 int a = 0, b = 0, c = 0;
	 
	 // positioning the Properties Window
	 if(dlg[0].x != 0){
		 b = dlg[0].x;
		 c = dlg[0].y;
		 for(a=0;a<22;a++){
			 dlg[a].x -= b;
			 dlg[a].y -= c;
		 }
	 }
	 
	 for(a=0;a<22;a++){
		 dlg[a].x += (Game::iRealScreenWidth/2) - (dlg[0].w/2);
		 dlg[a].y += (Game::iRealScreenHeight/2) - (dlg[0].h/2);
	 }
	 
	 // Setting-up the dlg_picview pointer
	 dlg_picview = create_bitmap(66, 50);
	 clear_to_color(dlg_picview, makecol(41, 145, 213));
	 text_mode(-1);
	 textprintf_centre(dlg_picview, font, dlg_picview->w/2, dlg_picview->h/2-4, makecol(255, 255, 255), "???");
	 dlg[17].dp = (void*)dlg_picview;
	 
	 // Setting-up the strings of the listboxes
	 d_listbox_idmusic_strs = new char*[ int(Data::music.size()) ];
	 for(a=0;a<int(Data::music.size());a++){
		 d_listbox_idmusic_strs[a] = new char[121];
		 sprintf(d_listbox_idmusic_strs[a], "%03d", a);
	 }
	 
	 d_listbox_idsuperbg_strs = new char*[ int(Data::superBgAnimation.size()) ];
	 for(a=0;a<int(Data::superBgAnimation.size());a++){
		 d_listbox_idsuperbg_strs[a] = new char[121];
		 sprintf(d_listbox_idsuperbg_strs[a], "%03d", a);
	 }
	 
	 d_listbox_idbg_strs = new char*[ int(Data::bgAnimation.size()) ];
	 for(a=0;a<int(Data::bgAnimation.size());a++){
		 d_listbox_idbg_strs[a] = new char[121];
		 sprintf(d_listbox_idbg_strs[a], "%03d", a);
	 }
	 
	 d_listbox_idfg_strs = new char*[ int(Data::fgAnimation.size()) ];
	 for(a=0;a<int(Data::fgAnimation.size());a++){
		 d_listbox_idfg_strs[a] = new char[121];
		 sprintf(d_listbox_idfg_strs[a], "%03d", a);
	 }
	 
	 // Loading some information
	 strcpy(dlg_stagename, LvMap.name.c_str());
	 dlg[19].flags = 0;
	 dlg[20].flags = 0;
	 dlg[21].flags = 0;
	 if(LvMap.win_conditions & WINC_FINDEXIT) dlg[19].flags |= D_SELECTED;
	 if(LvMap.win_conditions & WINC_KILLALL) dlg[20].flags |= D_SELECTED;
	 if(LvMap.win_conditions & WINC_KILLBOSS) dlg[21].flags |= D_SELECTED;
	 dlg[6].d1 = dlg[6].d2 = LvMap.iBg[0];
	 dlg[8].d1 = dlg[8].d2 = LvMap.iBg[1];
	 dlg[10].d1 = dlg[10].d2 = LvMap.iBg[2];
	 
	 // String table
	 for(a=0;a<20;a++)
		 strcpy(dlg_strtable[a], Data::Lang("levedit_dlg_str" + StringUtils::int2str(a)).c_str());
 }

 void levedit_d_releaseall(){
	 int a;
	 int dlg_winconditions, dlg_music, dlg_superbg, dlg_bg, dlg_fg;
	 
	 // unloading stuff
	 destroy_bitmap(dlg_picview);
	 
	 for(a=0;a<int(Data::music.size());a++) delete[] d_listbox_idmusic_strs[a];
	 delete[] d_listbox_idmusic_strs;
	 
	 for(a=0;a<int(Data::bgAnimation.size());a++) delete[] d_listbox_idsuperbg_strs[a];
	 delete[] d_listbox_idsuperbg_strs;
	 
	 for(a=0;a<int(Data::bgAnimation.size());a++) delete[] d_listbox_idbg_strs[a];
	 delete[] d_listbox_idbg_strs;
	 
	 for(a=0;a<int(Data::bgAnimation.size());a++) delete[] d_listbox_idfg_strs[a];
	 delete[] d_listbox_idfg_strs;
	 
	 // checking map info
	 dlg_music = dlg[4].d1;
	 dlg_superbg = dlg[6].d1;
	 dlg_bg = dlg[8].d1;
	 dlg_fg = dlg[10].d1;
	 dlg_winconditions = (dlg[19].flags & D_SELECTED) ? WINC_FINDEXIT : 0;
	 dlg_winconditions |= (dlg[20].flags & D_SELECTED) ? WINC_KILLALL : 0;
	 dlg_winconditions |= (dlg[21].flags & D_SELECTED) ? WINC_KILLBOSS : 0;
	 
	 if(dlg_winconditions == 0){
		 play_sample(Data::sample[5]);
		 alert(dlg_strtable[14], dlg_strtable[15], dlg_strtable[16], "OK", NULL, 0, 0);
		 dlg_winconditions |= WINC_FINDEXIT;
	 }
	 
	 if(*dlg_stagename == '\0'){
		 play_sample(Data::sample[5]);
		 alert(dlg_strtable[17], dlg_strtable[18], dlg_strtable[19], "OK", NULL, 0, 0);
		 strcpy(dlg_stagename, "Noname");
	 }
	 
	 // updating map information
	 LvMap.name = dlg_stagename;
	 LvMap.iMusic = dlg_music;
	 LvMap.iBg[0] = dlg_superbg;
	 LvMap.iBg[1] = dlg_bg;
	 LvMap.iBg[2] = dlg_fg;
	 LvMap.win_conditions = dlg_winconditions;
	 
 }
 
 void levedit_changeproperties(){
	 
	 levedit_d_loadall();
	 do_dialog(dlg, -1);
	 levedit_d_releaseall();
	 Timer::fix_speedcounter();
	 
 }
 
 // And now, the huge DIALOG thing
 DIALOG dlg[]={
	 /* -- */   /* (dialog proc)          (x)   (y)   (w)   (h)   (fg)     (bg)       (key) (flags)  (d1)   (d2)  (dp)                        (dp2) (dp3) */
	 /* 0  */   { d_agup_window_proc,     0,    0,    300,  220,  0,        0,         0,    0,       0,     0,    (void*)dlg_strtable[0],                       NULL, NULL  },
     /* 1  */   { d_agup_text_proc,       5,    25,   0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[1],     NULL, NULL  },
     /* 2  */   { d_agup_edit_proc,       115,  22,   175,  8,    0,        0,         0,    0,       30,    0,    dlg_stagename,              NULL, NULL  },
     /* 3  */   { d_agup_text_proc,       5,    50,   0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[2],     NULL, NULL  },
     /* 4  */   { d_agup_list_proc,       5,    62,   65,   75,   0,        0,         0,    0,       0,     0,    (void*)d_listbox_idmusic,   NULL, NULL  },
     /* 5  */   { d_agup_text_proc,       75,   50,   0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[3],     NULL, NULL  },
     /* 6  */   { d_agup_list_proc,       75,   62,   65,   75,   0,        0,         0,    0,       0,     0,    (void*)d_listbox_idsuperbg, NULL, NULL  },
     /* 7  */   { d_agup_text_proc,       145,  50,   0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[4],     NULL, NULL  },
     /* 8  */   { d_agup_list_proc,       145,  62,   65,   75,   0,        0,         0,    0,       0,     0,    (void*)d_listbox_idbg,      NULL, NULL  },
     /* 9  */   { d_agup_text_proc,       215,  50,   0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[5],     NULL, NULL  },
     /* 10 */   { d_agup_list_proc,       215,  62,   65,   75,   0,        0xEF7D,    0,    0,       0,     0,    (void*)d_listbox_idfg,      NULL, NULL  },
     /* 11 */   { d_agup_push_proc,       5,    145,  65,   15,   0,        0xA554,    0,    D_EXIT,  0,     0,    (void*)dlg_strtable[6],     NULL, (void*)d_playmusic },
     /* 12 */   { d_agup_push_proc,       75,   145,  65,   15,   0,        0xA554,    0,    D_EXIT,  0,     0,    (void*)dlg_strtable[7],     NULL, (void*)d_picviewer  },
     /* 13 */   { d_agup_push_proc,       145,  145,  65,   15,   0,        0xA554,    0,    D_EXIT,  1,     0,    (void*)dlg_strtable[7],     NULL, (void*)d_picviewer  },
     /* 14 */   { d_agup_push_proc,       215,  145,  65,   15,   0,        0xA554,    0,    D_EXIT,  2,     0,    (void*)dlg_strtable[7],     NULL, (void*)d_picviewer  },
     /* 15 */   { d_agup_button_proc,     255,  205,  40,   12,   0,        0xA554,    0,    D_EXIT,  0,     0,    (void*)"OK",                NULL, NULL  },
	 /* 16 */   { d_agup_box_proc,        4,    164,  68,   52,   0,        0,         0,    0,       0,     0,    NULL,                       NULL, NULL  },
	 /* 17 */   { d_bitmap_proc,          5,    165,  66,   50,   0,        0,         0,    0,       0,     0,    NULL,                       NULL, NULL  },
	 /* 18 */   { d_agup_text_proc,       80,   167,  0,    0,    0,        -1,        0,    0,       0,     0,    (void*)dlg_strtable[8],     NULL, NULL  },
	 /* 19 */   { d_agup_check_proc,      80,   180,  140,  10,   0,        -1,        0,    0,       1,     0,    (void*)dlg_strtable[9],     NULL, NULL  },
	 /* 20 */   { d_agup_check_proc,      80,   192,  205,  10,   0,        -1,        0,    0,       1,     0,    (void*)dlg_strtable[10],    NULL, NULL  },
	 /* 21 */   { d_agup_check_proc,      80,   204,  155,  10,   0,        -1,        0,    0,       1,     0,    (void*)dlg_strtable[11],    NULL, NULL  },
     /* -- */   { NULL,                   0,    0,    0,    0,    0,        0,         0,    0,       0,     0,    NULL,                       NULL, NULL  }
 };
