/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include "Game.h"
 
 // variables
 BITMAP					*Game::bmpBuffer = NULL;
 BITMAP					*Game::_bmp2ndBuffer = NULL;
 ALFONT_FONT			**Game::fntDebug = NULL;
 bool					Game::bDebug = false;
 bool					Game::bGetOut = false;
 bool					Game::bLoadingComplete = false;
 bool					Game::gfx_bFullscreen = false;
 int					Game::gfx_iRenderType = 0;
 int					Game::gfx_iRenderEngine = 0;
 int					Game::gfx_iBpp = 16;
 int					Game::iLanguage = -1;
 int					Game::iRealScreenWidth = 0;
 int					Game::iRealScreenHeight = 0;
 
 // << GUI-related >>
 // d_hackish_edit_proc(): from AGUP
 static int work_around_fsel_bug = 0;
 int d_hackish_edit_proc (int msg, DIALOG *d, int c){
     if (msg == MSG_START && !work_around_fsel_bug){
         // Adjust position/dimension so it is the same as AGUP's.
         d->y -= 2;
         d->h += 8;
         // The Allegro GUI has a bug where it repeatedely sends MSG_START to a
		 // custom GUI procedure. We need to work around that.
         work_around_fsel_bug = 1;
     }
     if (msg == MSG_END && work_around_fsel_bug){
         d->y += 2;
         d->h -= 8;
         work_around_fsel_bug = 0; 
     }
     return d_agup_edit_proc (msg, d, c);
 }
 
 // ---------------------
 // SDK functions & vars
 // ---------------------
 // play_sample(sample)
 int play_sample(SAMPLE *spl){
	 return play_sample(spl, 255, 128, 1000, false);
 }
 
 // get_hexdigit(char i)
 static int get_hexdigit(char i){
	if(i >= '0' && i <= '9')
		return i - '0';
	if(i >= 'A' && i <= 'F')
		return i - 'A' + 10;
	if(i >= 'a' && i <= 'f')
		return i - 'a' + 10;
	return 0;
 }

 // alfont_textprintf_color(dest, font, x, y, backg, wordwrap, msg, ...)
 bool alfont_textprintf_color_use_aa = true; // Use anti-aliasing?
 void alfont_textprintf_color(BITMAP* dest, ALFONT_FONT* font, int x, int y, int backg, int wordwrap, const char* msg, ...){
	static char buffer[5121], varcl[4];
	static int px, py, pos, color, ftext_length, ftext_height, wwc;
	static va_list va;
	
	px = x;
	py = y;
	pos = 0;
	color = makecol(0xFF, 0xFF, 0xFF);
	ftext_length = alfont_text_length(font, " ");
	ftext_height = alfont_text_height(font);
	wwc = 0; // wordwrap counter - counts all the chars on the line
	
	strcpy(buffer, "-");
	va_start(va, msg);
	vsprintf(buffer, msg, va);
	va_end(va);
	
	while(buffer[pos]){
		// -- hex colors --
		if(buffer[pos] == '$'){
			++pos;
			if(buffer[pos] != '$'){
				// "$rrggbb_"
				int r = 0, g = 0, b = 0;
				r = get_hexdigit(buffer[pos]) << 4;
				r |= get_hexdigit(buffer[pos + 1]);
				g = get_hexdigit(buffer[pos + 2]) << 4;
				g |= get_hexdigit(buffer[pos + 3]);
				b = get_hexdigit(buffer[pos + 4]) << 4;
				b |= get_hexdigit(buffer[pos + 5]);
				color = makecol(r, g, b);
				pos += 6;
				continue;
			}else ++wwc;
		}
		
		// -- var colors --
		// #wht   ,   #ylw   ,   #blu   ,   #red   ,   #grn   ,   #slv   ,   #vlt   ,   #blk
		// white  ,  yellow  ,   blue   ,   red    ,   green  ,  silver  ,  violet  ,   black
		if(buffer[pos] == '#'){
			++pos;
			if(buffer[pos] != '#'){
				
				// "#(color - 3 digits)"
				varcl[0] = buffer[pos];
				varcl[1] = buffer[pos+1];
				varcl[2] = buffer[pos+2];
				varcl[3] = '\0';
				
				// what's the color?
				if(strcmp(varcl, "wht") == 0) color = makecol(0xFF, 0xFF, 0xFF);
				else if(strcmp(varcl, "ylw") == 0) color = makecol(0xFF, 0xFF, 0x00);
				else if(strcmp(varcl, "blu") == 0) color = makecol(0x00, 0xCC, 0xFF);
				else if(strcmp(varcl, "red") == 0) color = makecol(0xFF, 0x00, 0x00);
				else if(strcmp(varcl, "grn") == 0) color = makecol(0x00, 0xDD, 0x00);
				else if(strcmp(varcl, "slv") == 0) color = makecol(0xAA, 0xAA, 0xAA);
				else if(strcmp(varcl, "vlt") == 0) color = makecol(0x80, 0x00, 0xFF);
				else if(strcmp(varcl, "blk") == 0) color = makecol(0x00, 0x00, 0x00);
				
				pos += 3;
			}else ++wwc;
		}
		
 		// -- tokens --
		if(buffer[pos] == '\n'){
			++pos;
			px = x;
			py += ftext_height;
			wwc = 0; // end of line
			continue;
		}
		if(buffer[pos] == '\\' && buffer[pos+1] == 'n'){
			pos += 2;
			px = x;
			py += ftext_height;
			wwc = 0; // end of line
			continue;
		}
		if(buffer[pos] == ' '){
			++pos; ++wwc;
			px += ftext_length;
			continue;
		}
		
		// -- wordwrap --
		if(wordwrap > 0){
			if(wwc >= wordwrap - 4 && isspace(buffer[pos-1])){
				px = x;
				py += ftext_height;
				wwc = 0; // end of line
				continue;
			}
		}
		
		// -- printing the text --
		char msg2[2] = " ";
		msg2[0] = buffer[pos];
		alfont_textprintf_color_use_aa ? alfont_textout_aa_ex(dest, font, msg2, px, py, color, backg) : alfont_textout_ex(dest, font, msg2, px, py, color, backg);
		px += alfont_text_length(font, msg2);
		++pos; ++wwc;
	}
 }
 
 // GSleep(msec)
 void GSleep(int x){
	 int end = clock()/(CLOCKS_PER_SEC/1000) + x;
	 while(clock()/(CLOCKS_PER_SEC/1000) < end){
		 while(Game::MayRun()){
			 Game::events();
		 }
		 Game::render();
	 }
	 Timer::fix_speedcounter();
 }
 
 // SuspendForKey()
 void SuspendForKey(){
	 clear_keybuf();
	 Player1_Control.wFlags = 0;
	 while(!keypressed() && Player1_Control.wFlags == 0){
		 while(Game::MayRun()){
			 Game::events();
		 }
		 Game::render();
	 }
	 Timer::fix_speedcounter();
 }
 
 // Bool2String(bool, type)
 string Bool2String(bool& b, unsigned char type){
	 static string str;
	 str = "Unkown type";
	 
	 switch(type){
	 case 0: // Yes / No
		 str = b ? "Yes" : "No";
		 break;
	 case 1: // True / False
		 str = b ? "Yes" : "No";
		 break;
	 }
	 
	 return str;
 }
 
 // rect_trans(bmp, x, y, width, height, color, transparency factor)
 void rect_trans(BITMAP* bmp, int x, int y, int w, int h, int color, int fact){
	 
	 #ifdef _USE_FBLEND
	 fblend_rect_trans(bmp, x, y, w, h, color, fact);
	 #else
	 static BITMAP* box;
	 box = create_bitmap(w, h);
	 clear_to_color(box, color);
	 set_trans_blender(0, 0, 0, fact);
	 draw_trans_sprite(bmp, box, x, y);
	 set_trans_blender(0, 0, 0, 0);
	 destroy_bitmap(box);
	 #endif
	 
 }
 
 // draw_trans(source, dest, x, y, transparency factor)
 void draw_trans(BITMAP* src, BITMAP* dest, int x, int y, int fact){
	 
	 #ifdef _USE_FBLEND
	 fblend_trans(src, dest, x, y, fact);
	 #else
	 set_trans_blender(0, 0, 0, fact);
	 draw_trans_sprite(dest, src, x, y);
	 set_trans_blender(0, 0, 0, 0);
	 #endif
	 
 }
 
 // SMAlert(msg)
 void SMAlert(string msg){
	 
	 int x = BUFFER_W/2 - Data::image[10]->w/2, y = BUFFER_H/2 - Data::image[10]->h/2;
	 
	 rect_trans(Game::bmpBuffer, x, y, Data::image[10]->w, Data::image[10]->h, 0, 175);
	 draw_sprite(Game::bmpBuffer, Data::image[10], x, y);
	 alfont_set_font_size(Data::font[_ARIAL5], 13);
	 alfont_textprintf_color(Game::bmpBuffer, Data::font[_ARIAL5], x+6, y+6, -1, 20, msg.c_str());
	 
	 // [OK]
	 alfont_textprintf_color(
		 Game::bmpBuffer,
		 Data::font[_ARIAL5],
		 x+Data::image[10]->w - alfont_text_length(Data::font[_ARIAL5],"[OK]")-6,
		 y+Data::image[10]->h - alfont_text_height(Data::font[_ARIAL5])-6,
		 -1, 0, "[OK]"
	 );
	 
	 play_sample(Data::sample[5]);
	 SuspendForKey();
	 
 }
 
 // MsgBox(msg): system Message Box
  void MsgBox(const char *fmt, ...){
	 
	 static char buf[5121];
	 static va_list ap;
	 
	 va_start(ap, fmt);
	 vsprintf(buf, fmt, ap);
	 va_end(ap);
	 
	 allegro_message(buf);
	 
 }
  
 void MsgBox(string msg){
	 MsgBox(msg.c_str());
 }
 
 // ---------------------------------------------------
 // events(): Main game events (runs every logic loop)
 // ---------------------------------------------------
 void Game::events(){
	 
	 // Timers
	 Timer::speedcounter--;
	 
	 // Polling...
	 Player1_Control.poll();
	 Player2_Control.poll();
	 poll_joystick();
	 if(keyboard_needs_poll())
		 poll_keyboard();
	 
	 // Debug mode?
	 if(bDebug && bLoadingComplete){
		 rectfill(bmpBuffer, 0, 0, BUFFER_W, 10, 0);
		 textprintf(bmpBuffer, font, 1, 1, makecol(255,255,255), 
			 "ver: %s, "
			 "fps: %d, "
			 "ren: %s",
			 SMWX2_VERSION,
			 getFps(),
			 getRenderEngineString().c_str()
		 );
	 }
	 
 }
 
 
 
 // ---------------------------------------------------
 // logfile(sMessage): adds a message to the logfile
 // ---------------------------------------------------
 void Game::logfile(const char *csMessage, ...){
	 
	 static bool firsttime = true;
	 ofstream fp;
	 time_t rawtime;
	 va_list lst;
	 char csReadyMessage[1024], csTime[1024];
	 int strlength, a;
	 
	 fp.open("./logfile.log",firsttime ? ios::trunc : ios::app);
	 if(!fp) return;
	 
	 // Default header
	 if(firsttime) fp << "-- Super Mario World X 2 Logfile --" << endl;
	 
	 // Preparing the Message
	 va_start(lst, csMessage);
	 vsprintf(csReadyMessage, csMessage, lst);
	 va_end(lst);
	 
	 // Configuring the csTime variable
	 firsttime = false;
	 time(&rawtime);
	 strcpy(csTime, ctime(&rawtime));
	 strlength = int(strlen(csTime));
	 for(a=0;a<strlength;a++){
		 if(csTime[a] == '\n')
			 csTime[a] = '\0';
	 }
	 
	 // Writing the Message
	 fp << csTime << ": " << csReadyMessage << endl;
	 fp.close();
	 
 }
 
 
 
 // ---------------------------------------------------
 // start(): starts the game
 // ---------------------------------------------------
 void Game::start(){
	 
	 CIni ini;
	 
	 // <general loader>
	 logfile("Logfile module initialized.");
	 logfile("Loading Allegro...");
	 set_uformat(U_ASCII);
	 allegro_init();
	 install_keyboard();
	 install_mouse();
	 
	 if(!exists("./nosound")){
		 if(install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) != 0)
			 logfile("Couldn't initialize sound module: %s", allegro_error);
	 }else logfile("...no sound");
	 
	 set_color_depth(gfx_iBpp);
	 if(start_gfx() != 0){
		 logfile("Couldn't set a graphic mode. Try restarting your computer.\nAborting... %s", allegro_error);
		 MsgBox("Cannot set a graphic mode. Try restarting your computer.\nAborting... %s", allegro_error);
		 exit(1);
	 }
	 Init_2xSaI(gfx_iBpp);
	 logfile("GFX ok! %dx%dx%d, %s, Fullscreen: %s", Game::iRealScreenWidth, Game::iRealScreenHeight, gfx_iBpp, getRenderEngineString().c_str(), YesNoString(gfx_bFullscreen).c_str());
	 
	 logfile("Initializing timers...");
	 Timer::setup();
	 logfile("Timers OK!");
	 
	 logfile("Loading FMOD...");
	 if(!exists("./nomusic")){
		 if(!FSOUND_Init(32000, 64, 0)){
			 logfile("Couldn't initialize FMOD");
			 MsgBox("Cannot initialize FMOD");
			 abort();
		 }
	 }else logfile("...no music");
	 logfile("Done!");
	 
	 logfile("Loading font modules...");
	 if(alfont_init() != ALFONT_OK){
		 logfile("Couldn't initialize font modules");
		 MsgBox("Cannot initialize font modules");
		 abort();
	 }
	 logfile("Done!");
	 
	 logfile("Getting the Language...");
	 if(!ini.ReadFile("./data/smwx2.ini")){
		 Game::iLanguage = -1;
		 ini.WriteInt("language", Game::iLanguage);
	 }else{
		 Game::iLanguage = ini.ReadInt("language");
	 }
	 logfile("Done!");
	 
	 logfile("Randomizing...");
	 srand(time(NULL));
	 logfile("Done!");
	 
	 logfile("The game has been initialized.");
	 // </general loader>
	 
	 // <AGUP loader>
	 logfile("Starting AGUP...");
	 agup_init(abeos_theme);
     gui_fg_color = agup_fg_color;
     gui_bg_color = agup_bg_color;
     gui_mg_color = agup_mg_color;
     gui_shadow_box_proc = d_agup_shadow_box_proc;
     gui_button_proc = d_agup_button_proc;
     gui_ctext_proc = d_agup_ctext_proc;
     gui_text_list_proc = d_agup_text_list_proc;
	 gui_edit_proc = d_hackish_edit_proc;
	 logfile("Done!");
	 // </AGUP loader>
	 
	 
	 
	 // <buffers>
	 logfile("Creating buffers...");
	 bmpBuffer = create_bitmap(BUFFER_W, BUFFER_H);
	 _bmp2ndBuffer = create_bitmap(640, 480);
	 clear_bitmap(bmpBuffer);
	 clear_bitmap(_bmp2ndBuffer);
	 logfile("Done!");
	 // </buffers>
	 
	 
	 
	 // <data loader>
	 logfile("Loading data...");
	 Data::loadAll();
	 logfile("All data has been loaded successfully!");
	 // </data loader>
	 
	 
	 // <language_selector>
	 if(Game::iLanguage < 0 || Game::iLanguage >= int(Data::lang.size())){
		 logfile("Language selector needs to be opened.");
		 Frame::language_selector();
		 logfile("Done! Selected language: \"%s\"", Data::lang[Game::iLanguage].language.c_str());
	 }
	 // </language_selector>
	 
	 
	 // Loading complete!
	 logfile("-- Game start --");
	 Timer::fix_speedcounter();
	 bLoadingComplete = true;
	 Game::bDebug = exists("./debugmode") ? true : false;
	 fntDebug = &Data::font[_ARIAL4];
	 alfont_set_font_size(*fntDebug, 12);
	 
 }
 
 
 // ---------------------------------------------------
 // end(): closes the game
 // ---------------------------------------------------
 void Game::end(){
	 
	 // Game end
	 logfile("-- Game end --");
	 
	 // <data unloader>
	 logfile("Unloading data...");
	 Data::releaseAll();
	 logfile("Done!");
	 // </data unloader>
	 
	 // <unloading buffers>
	 logfile("Deleting buffers...");
	 destroy_bitmap(bmpBuffer);
	 destroy_bitmap(_bmp2ndBuffer);
	 logfile("Done!");
	 // </unloading buffers>
	 
	 // <general unloader>
	 logfile("Closing font modules...");
	 alfont_exit();
	 logfile("Done!");
	 logfile("Closing FMOD...");
	 FSOUND_Close();
	 logfile("Done!");
	 logfile("Closing AGUP...");
	 agup_shutdown();
	 logfile("Done!");
	 logfile("Closing Allegro...");
	 allegro_exit();
	 logfile("Done!");
	 // </general unloader>
	 
 }
 
 
 // ---------------------------------------------------
 // start_gfx(): Sets the graphic mode
 // ---------------------------------------------------
 int Game::start_gfx(){
	 
	 CIni ini;
	 int wndmode, returnvalue;
	 
	 /*
	  * INI keys:
	  *   fullscreen = 0 or 1 (fullscreen mode?)
	  *   rendertype = 0 or 1 (normal - 320x240 or double-sized - 640x480)
	 */
	 
	 if(!ini.ReadFile("./data/smwx2.ini")){
		 ini.WriteInt("fullscreen", gfx_bFullscreen ? 1 : 0);
		 ini.WriteInt("rendertype", gfx_iRenderType);
		 ini.WriteInt("renderengine", gfx_iRenderEngine);
	 }else{
		 gfx_bFullscreen = ini.ReadInt("fullscreen") == 1 ? true : false;
		 gfx_iRenderType = ini.ReadInt("rendertype");
		 gfx_iRenderEngine = ini.ReadInt("renderengine");
	 }
	 
	 // wnd_mode (fullscreen or windowed)?
	 set_window_title(SMWX2_WINDOWTITLE);
	 wndmode = gfx_bFullscreen?GFX_AUTODETECT_FULLSCREEN:GFX_AUTODETECT_WINDOWED;
	 
	 // Render Type (normal or doublesized)?
	 switch(gfx_iRenderType){
	 case RENDERTYPE_NORMAL:
		 returnvalue = set_gfx_mode(wndmode, 320, 240, 0, 0);
		 Game::iRealScreenWidth = 320;
		 Game::iRealScreenHeight = 240;
		 break;
		 
	 case RENDERTYPE_DOUBLESIZED:
		 returnvalue = set_gfx_mode(wndmode, 640, 480, 0, 0);
		 Game::iRealScreenWidth = 640;
		 Game::iRealScreenHeight = 480;
		 break;
		 
	 default:
		 returnvalue = set_gfx_mode(wndmode, 640, 480, 0, 0);
		 Game::iRealScreenWidth = 640;
		 Game::iRealScreenHeight = 480;
		 break;
	 }
	 
	 // Render Engine (normal (fblend_2x_stretch()), 2xSaI or SuperEagle)?
	 if(gfx_iRenderEngine != RENDERENGINE_NORMAL
	 && gfx_iRenderEngine != RENDERENGINE_2XSAI
	 && gfx_iRenderEngine != RENDERENGINE_SUPEREAGLE
	 // && gfx_iRenderEngine != 3
	 )
		 gfx_iRenderEngine = RENDERENGINE_NORMAL;
	 
	 
	 // Checking stuff
	 if(returnvalue == 0) return 0;
	 else{
		 
		 // 1st step: try the double-sized one (640x480) in safe mode
		 logfile("Couldn't set the graphic mode (\"%s\"). Trying the double-sized one (safe mode - Normal (2x))...", allegro_error);
		 MsgBox("Cannot set the graphic mode (\"%s\"). Try choosing a different one next time.\nTrying the double-sized one (safe mode - Normal (2x))...", allegro_error);
		 returnvalue = set_gfx_mode(GFX_SAFE, 640, 480, 0, 0);
		 if(returnvalue == 0){
			 gfx_iRenderType = RENDERTYPE_DOUBLESIZED;
			 gfx_iRenderEngine = RENDERENGINE_NORMAL;
			 Game::iRealScreenWidth = 640;
			 Game::iRealScreenHeight = 480;
			 logfile("Double-Sized (safe) OK!");
			 return returnvalue;
		 }
		 
		 // 2nd step: try the safe one (320x240)
		 logfile("Couldn't set the graphic mode (\"%s\"). Trying the safe one (320x240)...", allegro_error);
		 MsgBox("Cannot set the graphic mode (\"%s\"). Trying the safe one (320x240)...", allegro_error);
		 returnvalue = set_gfx_mode(GFX_SAFE, 320, 240, 0, 0);
		 Game::iRealScreenWidth = 320;
		 Game::iRealScreenHeight = 240;
		 if(returnvalue != 0){
			 logfile("GFX_SAFE error.");
			 return returnvalue;
		 }
		 gfx_iRenderType = RENDERTYPE_NORMAL;
		 gfx_iRenderEngine = RENDERENGINE_NORMAL;
		 logfile("GFX_SAFE ok!");
		 return 0;
	 }
	 
	 return 0;
	 
 }
 
 
 
 // ---------------------------------------------------
 // abort(): Aborts the game
 // ---------------------------------------------------
 void Game::abort(){
	 
	 logfile("A fatal error has ocurred. Aborting...");
	 MsgBox("A fatal error has ocurred. Aborting...");
	 
	 destroy_bitmap(bmpBuffer);
	 FSOUND_Close();
	 alfont_exit();
	 allegro_exit();
	 
	 exit(1);
 }
 
 
 
 // ---------------------------------------------------
 // render(): Renders the screen
 // ---------------------------------------------------
 void Game::render(){
	 acquire_screen();
	 switch(gfx_iRenderType){
	 case RENDERTYPE_NORMAL:
		 blit(bmpBuffer, screen, 0, 0, 0, 0, BUFFER_W, BUFFER_H);
		 break;
		 
	 case RENDERTYPE_DOUBLESIZED:
		 switch(gfx_iRenderEngine){
		 case RENDERENGINE_NORMAL:
			 #ifdef _USE_FBLEND
			 fblend_2x_stretch(bmpBuffer, _bmp2ndBuffer, 0, 0, 0, 0, BUFFER_W, BUFFER_H);
			 #else
			 stretch_sprite(_bmp2ndBuffer, bmpBuffer, 0, 0, 640, 480);
			 #endif
			 break;
			 
		 case RENDERENGINE_2XSAI:
			 Super2xSaI(bmpBuffer, _bmp2ndBuffer, 0, 0, 0, 0, BUFFER_W, BUFFER_H);
			 break;
			 
		 case RENDERENGINE_SUPEREAGLE:
			 SuperEagle(bmpBuffer, _bmp2ndBuffer, 0, 0, 0, 0, BUFFER_W, BUFFER_H);
			 break;
			 
		 default:
			 stretch_sprite(_bmp2ndBuffer, bmpBuffer, 0, 0, 640, 480);
			 break;
		 }
		 blit(_bmp2ndBuffer, screen, 0, 0, 0, 0, _bmp2ndBuffer->w, _bmp2ndBuffer->h);
		 break;
		 
	 default:
		 blit(bmpBuffer, screen, 0, 0, 0, 0, BUFFER_W, BUFFER_H);
		 break;
	 }
	 release_screen();
	 Timer::iFpsCounter++;
 }
