/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #ifndef _GAME_H
 #define _GAME_H

 // game properties
 #define SMWX2_VERSION				"Demo 1"
 #define SMWX2_INTERNALVERSION		"SMWX2 Demo 1"
 #define SMWX2_WINDOWTITLE			"Super Mario World X 2 - Demo 1"
 
 // Code stuff
 #define _OS_WIN32
 //#define _OS_LINUX
 #define _USE_FBLEND
 
 // header files
 #include <allegro.h>
 #include <fmod.h>
 #include <alfont.h>
 #include <string>
 #include <fstream>
 #include <time.h>
 #include <stdarg.h>
 #include <math.h>
 #include "Timers.h"
 #include "Ini.h"
 #include "InputDevice.h"
 #include "Data.h"
 #include "FadeFX.h"
 
 #ifdef _USE_FBLEND
 #include <fblend.h>
 #endif
 
 // C headers
 extern "C"{
	 #include <agup.h>
	 #include <2xsai.h>
 }
 
 // frames
 #include "Frames.h"
 
 // namespace
 using namespace std;
 
 // globals
 #define BUFFER_W					320
 #define BUFFER_H					240
 #define RENDERTYPE_NORMAL			0
 #define RENDERTYPE_DOUBLESIZED		1
 
 // <only for 640x480 screen res>
 #define RENDERENGINE_NORMAL		0		// fblend_2x_stretch() / stretch_sprite()
 #define RENDERENGINE_2XSAI			1
 #define RENDERENGINE_SUPEREAGLE	2
 // </only for 640x480 screen res>
 
 #define UP							1
 #define RIGHT						2
 #define DOWN						3
 #define LEFT						4
 
 // SDK functions & vars
 void 			alfont_textprintf_color(BITMAP* dest, ALFONT_FONT* font, int x, int y, int backg, int wordwrap, const char *msg, ...);
 extern bool 	alfont_textprintf_color_use_aa;
 int 			play_sample(SAMPLE *spl);
 void 			GSleep(int x);
 void 			SuspendForKey();
 string 		Bool2String(bool& b, unsigned char type);
 void 			rect_trans(BITMAP* bmp, int x, int y, int w, int h, int color, int fact);
 void			draw_trans(BITMAP* src, BITMAP* dest, int x, int y, int fact);
 void			SMAlert(string msg);
 void			MsgBox(const char *fmt, ...);
 void			MsgBox(string msg);
 
 // Inline SDK functions
 inline string YesNoString(bool& b){ return Bool2String(b, 0); }
 inline string TrueFalseString(bool& b){ return Bool2String(b, 1); }
 
 // Macro properties
 //#define MGSLEEP_ENABLED
 
 // -- Macros --
 // MGSLEEP(x)
 #ifdef MGSLEEP_ENABLED
 #	define MGSLEEP(x)					GSleep(x)
 #else
 #	define MGSLEEP(x)
 #endif
 
 // Game class
 class Game{
 private:
	 
	 static BITMAP					*_bmp2ndBuffer;
	 
 public:
	 
	 static BITMAP					*bmpBuffer;
	 static ALFONT_FONT				**fntDebug;
	 static bool					bGetOut;
	 static bool					bDebug;
	 static bool					bLoadingComplete;
	 static bool					gfx_bFullscreen;
	 static int						gfx_iRenderType;
	 static int						gfx_iRenderEngine;
	 static int						gfx_iBpp;
	 static int						iLanguage;
	 static int						iRealScreenWidth;  // Can be 640 or 320
	 static int						iRealScreenHeight; // Can be 480 or 240
	 //static char					uBuf[10][10241]; // Unicode buffer - (10KB + 1 byte) * 10
	 
	 static void					logfile(const char *csMessage, ...);
	 static void					start();
	 static void					end();
	 static void					events();
	 static int						start_gfx();
	 static void					abort();	 
	 static void					render();
	 
	 static inline bool				MayRun(){ return Timer::speedcounter > 0; }
	 static inline int				getFps(){ return Timer::iFps; }
	 
	 // getRenderEngineString()
	 static inline string getRenderEngineString(){
		 if(gfx_iRenderType == RENDERTYPE_NORMAL) return "Normal";
		 else{
			 switch(gfx_iRenderEngine){
			 case RENDERENGINE_NORMAL: return "Normal (2x)";
			 case RENDERENGINE_2XSAI: return "2xSaI";
			 case RENDERENGINE_SUPEREAGLE: return "SuperEagle";
			 }
		 }
		 return "Stretched";
	 }
	 
 };
 
 #endif
