/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
#include "Ini.h"
#include "StringUtils.h"
#include <fstream>
#include <iostream>

// ------------------------------------
// ReadFile(sFile): reads a file... :)
// ------------------------------------
bool CIni::ReadFile(const string& sFile){
	
	ifstream fp(sFile.c_str());
	string sLine;
	vector<string> vLineStuff;
	
	sCurrentFile = sFile;
	data.clear();
	if(!fp)
		return false;
	
	while(!fp.eof()){
		// Getting & trimming
		getline(fp, sLine);
		cout << ""; // bugfix
		sLine = StringUtils::Trim(sLine);
		
		// Removing the \" characters
		while(sLine.find_first_of('\"') != string::npos)
			sLine.replace(sLine.find_first_of('\"'),1,"");
		
		// Splitting
		vLineStuff.clear();
		if(StringUtils::Split(sLine,"=",vLineStuff) != 0)
			data[vLineStuff[0]] = vLineStuff[1];
	}
	
	fp.close();
	return true;	
}



// ------------------------------------
// WriteString(sKey, sValue): writes a string and saves the INI
// ------------------------------------
bool CIni::WriteString(const string& sKey, const string& sValue){
	
	ofstream fp(sCurrentFile.c_str());
	string sLine;
	map<string,string>::iterator it;
	
	data[sKey] = sValue;
	
	if(!fp)
		return false;
	
	// Head
	fp << "; INI file: \"" << sCurrentFile.c_str() << "\""
	<< endl << "; ----------------------------------------"
	<< endl << endl;
	
	// Body
	for(it = data.begin(); it != data.end(); ++it){
		/* Is it a number? */ if( (atof(it->second.c_str()) != 0)  ||  (it->second == "0") ) sLine = it->first + " = " + it->second;
		/* Is it a string? */ else sLine = it->first + " = \"" + it->second + "\"";
		fp << sLine << endl;
	}
	
	// Done!
	fp.close();
	return true;
	
}



// ------------------------------------
// WriteInt(sKey, iValue): writes an int to the INI file
// ------------------------------------
bool CIni::WriteInt(const string& sKey, const int iValue){
	
	char csValue[100];
	sprintf(csValue,"%d",iValue);
	
	return WriteString(sKey,csValue);
	
}


// ------------------------------------
// WriteFloat(sKey, fValue): writes a float to the INI file
// ------------------------------------
bool CIni::WriteFloat(const string& sKey, const float fValue){
	
	char csValue[100];
	sprintf(csValue,"%f",fValue);
	
	return WriteString(sKey,csValue);
	
}



// ------------------------------------
// ReadStringVector(sKey, vVector): reads a vector from the INI file,
// using a ',' char as the delimiter
// ------------------------------------
int CIni::ReadStringVector(const string& sKey, vector<string>& vVector){
	
	string sValue = data[sKey];
	return StringUtils::Split(sValue,",",vVector);
	
}


// ------------------------------------
// ReadIntVector(sKey, vVector): reads a vector of ints
// ------------------------------------
int CIni::ReadIntVector(const string& sKey, vector<int>& vVector){
	
	vector<string> vStringVector;
	int a, iReturnValue;
	vVector.clear();
	
	iReturnValue = StringUtils::Split(data[sKey],",",vStringVector);
	for(a=0; a<int(vStringVector.size()); a++) vVector.push_back(atoi(vStringVector[a].c_str()));
	
	return iReturnValue;
	
}

// ------------------------------------
// ReadFloatVector(sKey, vVector): reads a vector of floats
// ------------------------------------
int CIni::ReadFloatVector(const string& sKey, vector<float>& vVector){
	
	vector<string> vStringVector;
	int a, iReturnValue;
	vVector.clear();
	
	iReturnValue = StringUtils::Split(data[sKey],",",vStringVector);
	for(a=0; a<int(vStringVector.size()); a++) vVector.push_back(atof(vStringVector[a].c_str()));
	
	return iReturnValue;
	
}
