/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include "InputDevice.h"
 
 // variables
 CInputDevice Player1_Control, Player2_Control;
 
 // ---------------------------------------------------
 // set_keys(...): Defines new keys to the current Input Device
 // ---------------------------------------------------
 void CInputDevice::set_keys(int kStart, int kLeft, int kRight, int kUp, int kDown, int kFire1, int kFire2){
	 
	 wFlags = 0;
	 iKey_start = kStart;
	 iKey_left = kLeft;
	 iKey_right = kRight;
	 iKey_up = kUp;
	 iKey_down = kDown;
	 iKey_fire1 = kFire1;
	 iKey_fire2 = kFire2;
	 
 }
 
 
 
 // ---------------------------------------------------
 // poll(): Polls the Input Device
 // ---------------------------------------------------
 void CInputDevice::poll(){
	 
	 wFlags = 0;
	 
	 if(bUseJoystick){
		 if(joy[0].stick[0].axis[1].d1) wFlags |= K_UP;
		 if(joy[0].stick[0].axis[1].d2) wFlags |= K_DOWN;
		 if(joy[0].stick[0].axis[0].d1) wFlags |= K_LEFT;
		 if(joy[0].stick[0].axis[0].d2) wFlags |= K_RIGHT;
		 if(joy[0].button[0].b) wFlags |= K_FIRE1;
		 if(joy[0].button[1].b) wFlags |= K_FIRE2;
		 if(key[KEY_ENTER]) wFlags |= K_START;
	 }
	 else
	 {
		 if(key[iKey_up]) wFlags |= K_UP;
		 if(key[iKey_down]) wFlags |= K_DOWN;
		 if(key[iKey_left]) wFlags |= K_LEFT;
		 if(key[iKey_right]) wFlags |= K_RIGHT;
		 if(key[iKey_fire1]) wFlags |= K_FIRE1;
		 if(key[iKey_fire2]) wFlags |= K_FIRE2;
		 if(key[iKey_start]) wFlags |= K_START;
	 }
	 
 }

 // ---------------------------------------------------
 // keypress(key): checks for an unique keypress event
 // (only one action then this event loops)
 // ---------------------------------------------------
 bool CInputDevice::keypress(int iKey){
	 
	 static int arrKey;
	 
	 switch(iKey){
	 case K_START:		arrKey = 0; break; // 1-0
	 case K_LEFT:		arrKey = 1; break; // 2-1
	 case K_RIGHT:		arrKey = 2; break; // 4-2
	 case K_UP:			arrKey = 3; break; // 8-3
	 case K_DOWN:		arrKey = 4; break; // 16-4
	 case K_FIRE1:		arrKey = 5; break; // 32-5
	 case K_FIRE2:		arrKey = 6; break; // 64-6
	 }
	 
	 if((wFlags & iKey) && !keypress_evbuff[arrKey]){
		 keypress_evbuff[arrKey] = true;
		 return true;
	 }
	 
	 if(!(wFlags & iKey)) keypress_evbuff[arrKey] = false;
	 return false;
	 
 }
