/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include <allegro.h>
 #include <winalleg.h>
 #include "Map.h"
 
 /////////// CMappos stuff ////////////////////////////////
 // operator = (int)
 void CMappos::operator=(int assign){
	 
	 tile[0] = tile[1] = tile[2] = assign;
	 mask = assign;
	 offset_x = offset_y = assign;
	 
 }
 
 // operator = (CMappos&)
 CMappos& CMappos::operator=(CMappos& src){
	 
	 tile[0]		 = 		src.tile[0];
	 tile[1]		 = 		src.tile[1];
	 tile[2]		 = 		src.tile[2];
	 mask			 = 		src.mask;
	 offset_x		 = 		src.offset_x;
	 offset_y		 = 		src.offset_y;
	 
	 return (*this);
	 
 }
 
 
 ////////////////// CMap stuff ///////////////////////////////
 // createnew()
 void CMap::createnew(int width, int height){
	 
	 CMappos cmappos_blankobj;
	 
	 name = "Noname";
	 iMusic = iBg[0] = iBg[1] = iBg[2] = 0;
	 win_conditions = WINC_FINDEXIT;
	 start_x = start_y = 0;
	 
	 tileData.resize(width, height);
	 tileData.clear(cmappos_blankobj);
	 itemData.clear();
	 enemyData.clear();
	 
 }
 
 // load(file, can read protected)
 bool CMap::load(const string& sFile, bool canReadProtected){
	 
	 PACKFILE *f = pack_fopen(sFile.c_str(), F_READ);
	 int a, b, x, y, currObjID, gotw;
	 char sign[9];
	 CSprite spr;
	 CMappos mpos;
	 bool protectedmap = false;
	 
	 Game::logfile("Preparing to open map \"%s\"...", sFile.c_str());
	 
	 if(!f){
		 Game::logfile("Couldn't load map \"%s\"", sFile.c_str());
		 return false;
	 }
	 
	 // Signature
	 for(a=0;a<8;a++) sign[a] = pack_getc(f);
	 sign[8] = 0;
	 
	 if(strcmp(sign, "SMWX2MAP") != 0){
		 Game::logfile("Couldn't load map \"%s\" - invalid map", sFile.c_str());
		 pack_fclose(f);
		 return false;
	 }else{
		 
		 // Is the map protected?
		 protectedmap = (pack_getc(f) == 'P');
		 if(protectedmap && !canReadProtected){
			 Game::logfile("Cannot read a protected map \"%s\"", sFile.c_str());
			 pack_fclose(f);
			 return false;
		 }
	 }
	 
	 // Map version
	 b = pack_getc(f);
	 if(b != MAP_VERSION){
		 Game::logfile("Incorrect map version (\"%s\"). It's %d when it should be %d.", sFile.c_str(), b, MAP_VERSION);
		 pack_fclose(f);
		 return false;
	 }
	 
	 // -----------------------
	 // Reading an unprotected map
	 // -----------------------
	 // 1. Properties
	 // 2. item data
	 // 3. enemy data
	 // 4. tile data (offsets, mask, tile[0-1-2])
	 if(!protectedmap){
		 
		 // -- 1. Properties --
		 // Map name
		 name = "";
		 b = pack_igetw(f);
		 for(a=0;a<b;a++) name += pack_getc(f);
		 
		 // Music & backgrounds
		 iMusic = pack_igetw(f);
		 for(a=0;a<3;a++) iBg[a] = pack_igetw(f);
		 
		 // Other variables
		 win_conditions = pack_getc(f);
		 start_x = pack_igetw(f);
		 start_y = pack_igetw(f);
		 
		 // -- 2. Items --
		 // Reading items
		 itemData.clear();
		 b = pack_igetw(f);
		 for(a=0;a<b;a++){
			 currObjID = pack_igetw(f);
			 spr.alterableValue[5] = currObjID;
			 spr.SetAnimPointer(&Data::item[currObjID]);
			 gotw = pack_igetw(f);
			 spr.x = gotw*32 + (16-Data::item[currObjID].getImage(0)->w/2);
			 gotw = pack_igetw(f);
			 spr.y = gotw*32 + (16-Data::item[currObjID].getImage(0)->h/2);
			 spr.interval[0] = Data::item[currObjID].animInterval[0][IANIM_NORMAL];
			 spr.interval[1] = Data::item[currObjID].animInterval[1][IANIM_NORMAL];
			 itemData.push_back(spr);
		 }
		 
		 // -- 3. Enemies --
		 // Reading
		 enemyData.clear();
		 b = pack_igetw(f);
		 for(a=0;a<b;a++){
			 currObjID = pack_igetw(f);
			 spr.alterableValue[5] = currObjID;
			 spr.SetAnimPointer(&Data::enemy[currObjID]);
			 gotw = pack_igetw(f);
			 spr.x = gotw*32 + (16-Data::enemy[currObjID].getImage(0)->w/2);
			 gotw = pack_igetw(f);
			 spr.y = gotw*32 + (32-Data::enemy[currObjID].getImage(0)->h) + 1;
			 spr.interval[0] = Data::enemy[currObjID].animInterval[0][EANIM_WALK];
			 spr.interval[1] = Data::enemy[currObjID].animInterval[1][EANIM_WALK];
			 enemyData.push_back(spr);
		 }
		 
		 // -- 4. Tile data --
		 // Reading (width, height)
		 a = pack_igetw(f);
		 b = pack_igetw(f);
		 tileData.resize(a, b);
		 tileData.clear(mpos);
		 
		 // Reading (offsets, mask, tile[0-1-2])
		 for(y=0;y<tileData.height();y++){
			 for(x=0;x<tileData.width();x++){
				 mpos.offset_x = pack_getc(f);
				 mpos.offset_y = pack_getc(f);
				 mpos.mask = pack_getc(f);
				 for(a=0;a<3;a++) mpos.tile[a] = pack_igetw(f);
				 tileData.set(x, y, mpos);
			 }
		 }
		 
	 }
	 
	 // -----------------------
	 // Reading a protected map
	 // -----------------------
	 // 1. enemy data
	 // 2. Properties
	 // 3. tile data (tile[2-1-0], mask, offsets)
	 // 4. item data
	 else{
		 
		 // -- 1. Enemies --
		 // Reading
		 enemyData.clear();
		 b = pack_igetw(f);
		 for(a=0;a<b;a++){
			 currObjID = pack_igetw(f);
			 spr.alterableValue[5] = currObjID;
			 spr.SetAnimPointer(&Data::enemy[currObjID]);
			 gotw = pack_igetw(f);
			 spr.x = gotw*32 + (16-Data::enemy[currObjID].getImage(0)->w/2);
			 gotw = pack_igetw(f);
			 spr.y = gotw*32 + (32-Data::enemy[currObjID].getImage(0)->h) + 1;
			 spr.interval[0] = Data::enemy[currObjID].animInterval[0][EANIM_WALK];
			 spr.interval[1] = Data::enemy[currObjID].animInterval[1][EANIM_WALK];
			 enemyData.push_back(spr);
		 }
		 
		 // -- 2. Properties --
		 // Map name
		 name = "";
		 b = pack_igetw(f);
		 for(a=0;a<b;a++) name += pack_getc(f);
		 
		 // Music & backgrounds
		 iMusic = pack_igetw(f);
		 for(a=0;a<3;a++) iBg[a] = pack_igetw(f);
		 
		 // Other variables
		 win_conditions = pack_getc(f);
		 start_x = pack_igetw(f);
		 start_y = pack_igetw(f);
		 
		 // -- 3. Tile data --
		 // Reading (width, height)
		 a = pack_igetw(f);
		 b = pack_igetw(f);
		 tileData.resize(a, b);
		 tileData.clear(mpos);
		 
		 // Reading (offsets, mask, tile[0-1-2])
		 for(y=0;y<tileData.height();y++){
			 for(x=0;x<tileData.width();x++){
				 for(a=2;a>=0;a++) mpos.tile[a] = pack_igetw(f);
				 mpos.mask = pack_getc(f);
				 mpos.offset_x = pack_getc(f);
				 mpos.offset_y = pack_getc(f);
				 tileData.set(x, y, mpos);
			 }
		 }
		 
		 // -- 4. Items --
		 // Reading items
		 itemData.clear();
		 b = pack_igetw(f);
		 for(a=0;a<b;a++){
			 currObjID = pack_igetw(f);
			 spr.alterableValue[5] = currObjID;
			 spr.SetAnimPointer(&Data::item[currObjID]);
			 gotw = pack_igetw(f);
			 spr.x = gotw*32 + (16-Data::item[currObjID].getImage(0)->w/2);
			 gotw = pack_igetw(f);
			 spr.y = gotw*32 + (16-Data::item[currObjID].getImage(0)->h/2);
			 spr.interval[0] = Data::item[currObjID].animInterval[0][IANIM_NORMAL];
			 spr.interval[1] = Data::item[currObjID].animInterval[1][IANIM_NORMAL];
			 itemData.push_back(spr);
		 }
		 
	 }
	 
	 Game::logfile("Map \"%s\" has been loaded (protected = %s)", sFile.c_str(), YesNoString(protectedmap).c_str());
	 pack_fclose(f);
	 return true;
 }

 
 // save(file, save protected)
 bool CMap::save(const string& sFile, bool saveProtected){
	 
	 PACKFILE *f = pack_fopen(sFile.c_str(), F_WRITE);
	 int a, x, y;
	 
	 Game::logfile("Saving map \"%s\" (protected = %s)...", sFile.c_str(), YesNoString(saveProtected).c_str());
	 
	 if(!f){
		 Game::logfile("Error: couldn't open file \"%s\" for writing", sFile.c_str());
		 return false;
	 }
	 
	 // Signature
	 char sign[9] = "SMWX2MAP";
	 sign[8] = saveProtected ? 'P' : 'p';
	 for(a=0;a<9;a++) pack_putc(sign[a], f);
	 
	 // Map version
	 pack_putc(MAP_VERSION, f);
	 
	 // -----------------------
	 // Saving an unprotected map
	 // -----------------------
	 // 1. Properties
	 // 2. item data
	 // 3. enemy data
	 // 4. tile data (offsets, mask, tile[0-1-2])
	 if(!saveProtected){
		 
		 // -- 1. Properties --
		 // Map name
		 pack_iputw(int(name.length()), f);
		 for(a=0;a<int(name.length());a++) pack_putc(name[a], f);
		 
		 // Music & backgrounds
		 pack_iputw(iMusic, f);
		 for(a=0;a<3;a++) pack_iputw(iBg[a], f);
		 
		 // Other variables
		 pack_putc(win_conditions, f);
		 pack_iputw(start_x, f);
		 pack_iputw(start_y, f);
		 
		 // -- 2. Items --
		 // Writing items
		 pack_iputw(int(itemData.size()), f);
		 for(a=0;a<int(itemData.size());a++){
			 pack_iputw(int(itemData[a].alterableValue[5]), f);
			 pack_iputw(int(itemData[a].x/32), f);
			 pack_iputw(int(itemData[a].y/32), f);
		 }
		 
		 // -- 3. Enemies --
		 // Writing
		 pack_iputw(int(enemyData.size()), f);
		 for(a=0;a<int(enemyData.size());a++){
			 pack_iputw(int(enemyData[a].alterableValue[5]), f);
			 pack_iputw(int(enemyData[a].x/32), f);
			 pack_iputw(int(enemyData[a].y/32), f);
		 }
		 
		 // -- 4. Tile data --
		 // Writing (width, height)
		 pack_iputw(tileData.width(), f);
		 pack_iputw(tileData.height(), f);
		 
		 // Writing (offsets, mask, tile[0-1-2])
		 for(y=0;y<tileData.height();y++){
			 for(x=0;x<tileData.width();x++){
				 pack_putc(tileData.pGet(x, y)->offset_x, f);
				 pack_putc(tileData.pGet(x, y)->offset_y, f);
				 pack_putc(tileData.pGet(x, y)->mask, f);
				 for(a=0;a<3;a++) pack_iputw(tileData.pGet(x, y)->tile[a], f);
			 }
		 }
		 
	 }
	 
	 // -----------------------
	 // Saving a protected map
	 // -----------------------
	 // 1. enemy data
	 // 2. Properties
	 // 3. tile data (tile[2-1-0], mask, offsets)
	 // 4. item data
	 else{
		 
		 // -- 1. Enemies --
		 // Writing
		 pack_iputw(int(enemyData.size()), f);
		 for(a=0;a<int(enemyData.size());a++){
			 pack_iputw(int(enemyData[a].alterableValue[5]), f);
			 pack_iputw(int(enemyData[a].x/32), f);
			 pack_iputw(int(enemyData[a].y/32), f);
		 }
		 
		 // -- 2. Properties --
		 // Map name
		 pack_iputw(int(name.length()), f);
		 for(a=0;a<int(name.length());a++) pack_putc(name[a], f);
		 
		 // Music & backgrounds
		 pack_iputw(iMusic, f);
		 for(a=0;a<3;a++) pack_iputw(iBg[a], f);
		 
		 // Other variables
		 pack_putc(win_conditions, f);
		 pack_iputw(start_x, f);
		 pack_iputw(start_y, f);
		 
		 // -- 3. Tile data --
		 // Writing (width, height)
		 pack_iputw(tileData.width(), f);
		 pack_iputw(tileData.height(), f);
		 
		 // Writing (offsets, mask, tile[2-1-0])
		 for(y=0;y<tileData.height();y++){
			 for(x=0;x<tileData.width();x++){
				 for(a=0;a<2;a++) pack_iputw(tileData.pGet(x, y)->tile[2-a], f);
				 pack_putc(tileData.pGet(x, y)->mask, f);
				 pack_putc(tileData.pGet(x, y)->offset_x, f);
				 pack_putc(tileData.pGet(x, y)->offset_y, f);
			 }
		 }
		 
		 // -- 4. Items --
		 // Writing items
		 pack_iputw(int(itemData.size()), f);
		 for(a=0;a<int(itemData.size());a++){
			 pack_iputw(int(itemData[a].alterableValue[5]), f);
			 pack_iputw(int(itemData[a].x/32), f);
			 pack_iputw(int(itemData[a].y/32), f);
		 }
		 
	 }
	 
	 Game::logfile("Map \"%s\" saved (protected = %s)", sFile.c_str(), YesNoString(saveProtected).c_str());
	 pack_fclose(f);
	 return true;
 }
