/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #ifndef _MAP_H
 #define _MAP_H
 
 // header files
 #include <string>
 #include <vector>
 #include "Game.h"
 #include "Utils.h"
 using namespace std;
 
 // -- map header --
 // Each time we modify the Map Header, the Structure,
 // we MUST modify its version.
 #define MAP_VERSION					1
 
 // map win conditions
 #define WINC_FINDEXIT					1
 #define WINC_KILLALL					2
 #define WINC_KILLBOSS					4
 
 // tile masks
 #define TMASK_NONE						0
 #define TMASK_FULL						1
 #define TMASK_DEATH					2
 #define TMASK_COUNT					3
 
 // map actions
 #define MACT_TELEPORT					0
 #define MACT_DOOR						1
 
 // CMappos
 class CMappos{
 public:
	 
	 unsigned short		tile[3];			// 3 layers (super-bg, bg, fg)
	 unsigned char		mask;				// no-mask, full-mask, death, etc.
	 char				offset_x, offset_y;	// offsets (type: signed char)
	 
	 CMappos(int assign = 0){ (*this) = assign; }
	 CMappos& operator=(CMappos& src);
	 void operator=(int assign);
	 
 };
 
 // CMapaction
 class CMapaction{
 public:
	 
	 unsigned char		action;		// a byte containing the action
	 unsigned short		value[5];	// 5 values (type: word)
	 
 };
 
 // CMap
 class CMap{
 public:
	 
	 string				name;
	 int				iMusic, iBg[3]; // iBg[0]: superbg , iBg[1]: bg , iBg[2]: fg
	 unsigned char		win_conditions;
	 int				start_x, start_y;
	 C2DArray<CMappos>	tileData;
	 vector<CSprite>	itemData;
	 vector<CSprite>	enemyData;
	 
	 void				createnew(int width, int height);
	 bool				load(const string& sFile, bool canReadProtected = false);
	 bool				save(const string& sFile, bool saveProtected = false);
	 
	 inline int			width(){ return tileData.width(); }
	 inline int			height(){ return tileData.height(); }
	 
	 CMap(){ createnew(10, 8); }
	 
 };
 
 #endif
