/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #include "Sprite.h"
 
 
 //////////////////////// CAnimation stuff //////////////////////
 // ---------------------------------------------------
 // import(): Imports an animation from a file
 // ---------------------------------------------------
 void CAnimation::import(const string& sFilePath){
	 
	 CIni Ini;
	 string str, temp;
	 vector<string> splitted;
	 int a;
	 
	 if(!Ini.ReadFile(sFilePath)){
		 Game::logfile("Animation error: Impossible to read INI file \"%s\"",sFilePath.c_str());
		 MsgBox("Animation error: Impossible to read INI file \"%s\"",sFilePath.c_str());
		 return;
	 }
	 
	 animSpeed = Ini.ReadInt("animSpeed");
	 if(animSpeed <= 0) animSpeed = 10;
	 str = Ini.ReadString("animation");
	 StringUtils::Split(str,"%",splitted);
	 
	 for(a=0;;a++){
		 temp = splitted[0] + StringUtils::int2str(a) + splitted[1];
		 if(exists(temp.c_str())) images.push_back(load_bitmap(temp.c_str(),NULL));
		 else break;
	 }
	 
	 if(a==0){
		 Game::logfile("Impossible to read animation (\"%s\") frame - file \"%s\" doesn't exist", sFilePath.c_str(), temp.c_str());
		 MsgBox("Impossible to read animation frame - file doesn't exist\n\nAnimation def file: \"%s\"\nAnimation image: \"%s\"", sFilePath.c_str(), temp.c_str());
		 Game::abort();
	 }
	 
 }
 
 // ---------------------------------------------------
 // importMapTile(): imports a map tile.
 // For example:
 // importMapTile(0,x) imports 0_0.pcx ,
 // importMapTile(1,x) imports 1_0.pcx , 1_1.pcx , 1_2.pcx ,
 // importMapTile(2,x) imports 2_0.pcx ,
 // importMapTile(3,x) imports 3_0.pcx, 3_1.pcx , ... , 3_20.pcx
 // ---------------------------------------------------
 void CAnimation::importMapTile(int tileID){
	 
	 static int a;
	 static string file;
	 
	 if(animSpeed <= 0) animSpeed = 10;
	 for(a=0;;a++){
		 file = "./data/maptiles/" + StringUtils::int2str(tileID) + "_" + StringUtils::int2str(a) + ".pcx";
		 if(exists(file.c_str())) images.push_back(load_bitmap(file.c_str(), NULL));
		 else break;
	 }
	 
	 if(a==0){
		 Game::logfile("Impossible to read mapTile (ID: %d) - file doesn't exist", tileID);
		 MsgBox("Impossible to read mapTile (ID: %d) - file doesn't exist", tileID);
		 Game::abort();
	 }
	 
 }
 
 
 //////////////////////// CSprite stuff //////////////////////
 // ---------------------------------------------------
 // draw(bmpDest): draws the sprite to a bitmap
 // ---------------------------------------------------
 void CSprite::draw(BITMAP *bmpDest){
	 
	 static BITMAP *bmpTemp = NULL;
	 
	 if(anim){
		 if(!(flags & SPRITE_INVISIBLE)){
			 
			 if(transparency == 0){
				 
				 if((flags & SPRITE_HFLIP) && (flags & SPRITE_VFLIP))
					 draw_sprite_vh_flip(bmpDest, anim->getImage(currentFrame), int(x), int(y));
				 else if(flags & SPRITE_HFLIP)
					 draw_sprite_h_flip(bmpDest, anim->getImage(currentFrame), int(x), int(y));
				 else if(flags & SPRITE_VFLIP)
					 draw_sprite_v_flip(bmpDest, anim->getImage(currentFrame), int(x), int(y));
				 else
					 draw_sprite(bmpDest, anim->getImage(currentFrame), int(x), int(y));
				 
			 }else{
				 
				 bmpTemp = create_bitmap(anim->getImage(currentFrame)->w, anim->getImage(currentFrame)->h);
				 clear_to_color(bmpTemp, makecol(255, 0, 255));
				 
				 if((flags & SPRITE_HFLIP) && (flags & SPRITE_VFLIP))
					 draw_sprite_vh_flip(bmpTemp, anim->getImage(currentFrame), 0, 0);
				 else if(flags & SPRITE_HFLIP)
					 draw_sprite_h_flip(bmpTemp, anim->getImage(currentFrame), 0, 0);
				 else if(flags & SPRITE_VFLIP)
					 draw_sprite_v_flip(bmpTemp, anim->getImage(currentFrame), 0, 0);
				 else
					 draw_sprite(bmpTemp, anim->getImage(currentFrame), 0, 0);
				 
				 #ifdef _USE_FBLEND
				 fblend_trans(bmpTemp, bmpDest, int(x), int(y), transparency);
				 #else
				 set_trans_blender(0,0,0,transparency);
				 draw_trans_sprite(bmpDest, bmpTemp, int(x), int(y));
				 set_trans_blender(0,0,0,0);
				 #endif
				 
				 destroy_bitmap(bmpTemp);
				 bmpTemp = NULL;
			 }
			 
		 }
	 }
	 
 }

 
 
 
 
 
 //////////////////////// CPlayer stuff //////////////////////
 // ---------------------------------------------------
 // import(sFilePath): imports a player from a file
 // ---------------------------------------------------
 void CPlayer::import(const string& sFilePath){
	 
	 int a;
	 CIni Ini;
	 vector<int> v;
	 string intervals[ANIM_COUNT] = {"animStop", "animWalk", "animJump", "animFall", "animDefeat", "animCrouchdown", "animSwim" };
	 
	 if(!Ini.ReadFile(sFilePath)){
		 Game::logfile("CPlayer::import() : Couldn't open file \"%s\"", sFilePath.c_str());
		 MsgBox("CPlayer::import() : Couldn't open file \"%s\"", sFilePath.c_str());
		 Game::abort();
	 }
	 
	 // reading misc properties
	 name = Ini.ReadString("name");
	 maxSpeed = Ini.ReadFloat("maxSpeed");
	 jumpStrength = Ini.ReadFloat("jumpStrength");
	 gravity = Ini.ReadFloat("gravity");
	 acceleration = Ini.ReadFloat("acceleration");
	 deceleration = Ini.ReadFloat("deceleration");
	 
	 if(name == "") name = "Noname";
	 if(maxSpeed <= 0.0f) maxSpeed = 1.0f;
	 if(jumpStrength <= 0.0f) jumpStrength = 1.0f;
	 if(gravity <= 0.0f) gravity = 1.0f;
	 if(acceleration < 0.0f) acceleration = 0.0f;
	 if(deceleration < 0.0f) deceleration = 0.0f;
	 
	 // reading the intervals
	 for(a=0;a<ANIM_COUNT;a++){
		 Ini.ReadIntVector(intervals[a], v);
		 if(v.size() != 2){
			 Game::logfile("CPlayer::import() : error - incorrect interval syntax on file \"%s\" , key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 MsgBox("CPlayer::import() : error - incorrect interval syntax on file \"%s\" , key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 Game::abort();
		 }
		 animInterval[0][a] = v[0];
		 animInterval[1][a] = v[1];
	 }
	 
	 // reading the animation
	 anim = new CAnimation(sFilePath);
	 ChangeAnimation(ANIM_STOP);
	 
 }

 
 // ---------------------------------------------------
 // release(): releases a loaded player
 // ---------------------------------------------------
 void CPlayer::release(){
	 
	 int a;
	 
	 // releasing the animation
	 anim->release();
	 delete anim;
	 
	 // now, the properties...
	 name = "";
	 maxSpeed = 0.0f;
	 jumpStrength = 0.0f;
	 gravity = 0.0f;
	 acceleration = 0.0f;
	 deceleration = 0.0f;
	 for(a=0;a<ANIM_COUNT;a++){
		 animInterval[0][a] = 0;
		 animInterval[1][a] = 0;
	 }
	 
 }

 
 // ---------------------------------------------------
 // getChar(sCharName): returns a pointer to a CPlayer
 // class - and this class contain the specified character
 // ---------------------------------------------------
 CPlayer* CPlayer::getChar(string sCharName){
	 
	 vector<CPlayer>::iterator it;
	 for(it = Data::character.begin(); it != Data::character.end(); ++it){
		 if(it->name == sCharName) return (CPlayer*)&*it;
	 }
	 
	 return (CPlayer*)&*Data::character.begin();
 }

 
 
 
 
 
 
 
 
 
 
 
 
 //////////////////////// CEnemy stuff //////////////////////
 // ---------------------------------------------------
 // eRead(sFilePath): reads some enemy's information
 //   (like name, moveSpeed, score).
 //   This function does NOT load any animation.
 // ---------------------------------------------------
 void CEnemy::eRead(const string& sFilePath){
	 
	 CIni Ini;
	 string intervals[EANIM_COUNT] = { "animStop" , "animWalk" , "animDefeat" };
	 vector<int> v;
	 int a;
	 
	 if(!Ini.ReadFile(sFilePath)){
		 Game::logfile("CEnemy::eRead() : Couldn't open file \"%s\"", sFilePath.c_str());
		 MsgBox("CEnemy::eRead() : Cannot open file \"%s\"", sFilePath.c_str());
		 Game::abort();
	 }
	 
	 name = Ini.ReadString("name");
	 moveSpeed = Ini.ReadFloat("moveSpeed");
	 score = Ini.ReadInt("score");
	 if(name == "") name = "Unknown";
	 if(moveSpeed <= 0) moveSpeed = 1;
	 if(score <= 0) score = 1;
	 
	 for(a=0;a<EANIM_COUNT;a++){
		 Ini.ReadIntVector(intervals[a], v);
		 if(v.size() != 2){
			 Game::logfile("CEnemy::eRead() : Incorrect interval syntax on file \"%s\", key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 MsgBox("CEnemy::eRead() : Incorrect interval syntax on file \"%s\", key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 Game::abort();
		 }
		 
		 animInterval[0][a] = v[0];
		 animInterval[1][a] = v[1];
	 }
	 
 };
 
 
 
 // ---------------------------------------------------
 // getEnemy(sEnemyName): returns a pointer to a CEnemy
 // class - and this class contain the specified enemy
 // ---------------------------------------------------
 CEnemy* CEnemy::getEnemy(string sEnemyName){
	 
	 vector<CEnemy>::iterator it;
	 for(it = Data::enemy.begin(); it != Data::enemy.end(); ++it){
		 if(it->name == sEnemyName) return (CEnemy*)&*it;
	 }
	 
	 return (CEnemy*)&*Data::enemy.begin();
	 
 }

 
 
 
 
 
 //////////////////////// CItem stuff //////////////////////
 // ---------------------------------------------------
 // eRead(sFilePath): reads some item's information
 //   (like name).
 //   This function does NOT load any animation.
 // ---------------------------------------------------
 void CItem::eRead(const string& sFilePath){
	 
	 CIni Ini;
	 string intervals[IANIM_COUNT] = { "animNormal" , "animTouched" };
	 vector<int> v;
	 int a;
	 
	 if(!Ini.ReadFile(sFilePath)){
		 Game::logfile("CItem::eRead() : Couldn't open file \"%s\"", sFilePath.c_str());
		 MsgBox("CItem::eRead() : Cannot open file \"%s\"", sFilePath.c_str());
		 Game::abort();
	 }
	 
	 name = Ini.ReadString("name");
	 score = Ini.ReadInt("score");
	 if(name == "") name = "Noname";
	 if(score < 0) score = 0;
	 
	 for(a=0;a<IANIM_COUNT;a++){
		 Ini.ReadIntVector(intervals[a], v);
		 if(v.size() != 2){
			 Game::logfile("CItem::eRead() : Incorrect interval syntax on file \"%s\", key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 MsgBox("CItem::eRead() : Incorrect interval syntax on file \"%s\", key = \"%s\"", sFilePath.c_str(), intervals[a].c_str());
			 Game::abort();
		 }
		 
		 animInterval[0][a] = v[0];
		 animInterval[1][a] = v[1];
	 }
	 
 };
 
 
 
 // ---------------------------------------------------
 // getItem(sEnemyName): returns a pointer to a CItem
 // class - and this class contain the specified item
 // ---------------------------------------------------
 CItem* CItem::getItem(string sItemName){
	 
	 vector<CItem>::iterator it;
	 for(it = Data::item.begin(); it != Data::item.end(); ++it){
		 if(it->name == sItemName) return (CItem*)&*it;
	 }
	 
	 return (CItem*)&*Data::item.begin();
	 
 }
