/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #ifndef _SPRITE_H
 #define _SPRITE_H
 
 
 // Sprite stuff
 #define SPRITE_NORMAL				0
 #define SPRITE_INVISIBLE			1
 #define SPRITE_HFLIP				2
 #define SPRITE_VFLIP				4
 
 // Player stuff
 #define ANIM_STOP					0
 #define ANIM_WALK					1
 #define ANIM_JUMP					2
 #define ANIM_FALL					3
 #define ANIM_DEFEAT				4
 #define ANIM_CROUCHDOWN			5
 #define ANIM_SWIM					6
 #define ANIM_COUNT					7
 
 // Enemy stuff
 #define EANIM_STOP					0
 #define EANIM_WALK					1
 #define EANIM_DEFEAT				2
 #define EANIM_COUNT				3
 
 // Item stuff
 #define IANIM_NORMAL				0
 #define IANIM_TOUCHED				1
 #define IANIM_COUNT				2
 
 
 
 #include <allegro.h>
 #include <vector>
 #include <string>
 #include "Game.h"
 #include "Ini.h"
 #include "StringUtils.h"
 #include "Data.h"
 using namespace std;

 // --------------
 // CAnimation
 // --------------
 class CAnimation{
 friend class CSprite;
 private:
	 vector<BITMAP*>	images;
	 
 public:
	 int				animSpeed;
	 
	 void				import(const string& sFilePath);
	 void				importMapTile(int tileID);
	 
	 // CAnimation()
	 CAnimation(){
		 animSpeed = 10;
	 }
	 
	 // CAnimation(file)
	 CAnimation(const string& sFilePath){
		 animSpeed = 10;
		 import(sFilePath);
	 }
	 
	 // CAnimation(tileID) - for mapTiles
	 CAnimation(int tileID){
		 animSpeed = 10;
		 importMapTile(tileID);
	 }
	 
	 // getImage(currentFrame)
	 inline BITMAP* getImage(int currentFrame){
		 
		 if(currentFrame < 0 || currentFrame >= int(images.size())){
			 currentFrame = 0;
		 }
		 
		 return images[currentFrame];
		 
	 }
	 
	 // release()
	 inline void release(){
		 
		 vector<BITMAP*>::iterator it;
		 for(it = images.begin(); it != images.end(); ++it){
			 destroy_bitmap((*it));
		 }
		 
		 animSpeed = 0;
		 
	 }
	 
 };
 
 
 // --------------
 // CSprite
 // --------------
 class CSprite{
 private:
	 int			_fLoop;
	 
 public:
	 float			x;
	 float			y;
	 float			alterableValue[6]; // DON'T TOUCH ON alterableValue[5]!
	 unsigned char	transparency;
	 unsigned int	flags;
	 
	 CAnimation		*anim;
	 int			currentFrame;
	 int			interval[2];
	 
	 
	 void			draw(BITMAP *bmpDest);
	 
	 // CSprite()
	 CSprite(){
		 x = 0.0f;
		 y = 0.0f;
		 alterableValue[0] = 0.0f;
		 alterableValue[1] = 0.0f;
		 alterableValue[2] = 0.0f;
		 alterableValue[3] = 0.0f;
		 alterableValue[4] = 0.0f;
		 alterableValue[5] = 0.0f;
		 transparency = 0;
		 flags = 0;
		 currentFrame = 0;
		 _fLoop = 0;
		 interval[0] = 0;
		 interval[1] = 0;
		 anim = NULL;
	 }
	 
	 // CSprite(animation)
	 CSprite(CAnimation *refAnim){
		 x = 0.0f;
		 y = 0.0f;
		 alterableValue[0] = 0.0f;
		 alterableValue[1] = 0.0f;
		 alterableValue[2] = 0.0f;
		 alterableValue[3] = 0.0f;
		 alterableValue[4] = 0.0f;
		 alterableValue[5] = 0.0f;
		 transparency = 0;
		 flags = 0;
		 currentFrame = 0;
		 _fLoop = 0;
		 anim = refAnim;
		 
		 if(anim){
			 interval[0] = 0;
			 interval[1] = int(anim->images.size() - 1);
		 }
	 }
	 
	 // width()
	 inline int width(){
		 if(!anim) return 0;
		 else return anim->getImage(currentFrame)->w;
	 }
	 
	 // height()
	 inline int height(){
		 if(!anim) return 0;
		 else return anim->getImage(currentFrame)->h;
	 }
	 
	 // SetAnimPointer(anim)
	 inline void SetAnimPointer(CAnimation *refAnim){
		 
		 anim = refAnim;
		 if(anim){
			 interval[0] = 0;
			 interval[1] = int(anim->images.size() -1);
		 }
		 currentFrame = interval[0];
		 _fLoop = 0;
		 
	 }
	 
	 // RunAnimation()
	 inline void RunAnimation(){
		 if(anim){
			 if(++_fLoop >= anim->animSpeed){
				 _fLoop = 0;
				 
				 if(currentFrame < interval[0]){
					 currentFrame = interval[0];
				 }
				 
				 if(++currentFrame > interval[1]){
					 currentFrame = interval[0];
				 }
			 }
		 }
	 }
	 
	 // hflipped() & vflipped()
	 inline bool hflipped(){
		 return flags & SPRITE_HFLIP ? true : false;
	 }
	 
	 inline bool vflipped(){
		 return flags & SPRITE_VFLIP ? true : false;
	 }
	 
	 // hflip() & vflip()
	 inline void hflip(){
		 if(flags & SPRITE_HFLIP) flags ^= SPRITE_HFLIP;
		 else flags |= SPRITE_HFLIP;
	 }
	 
	 inline void vflip(){
		 if(flags & SPRITE_VFLIP) flags ^= SPRITE_VFLIP;
		 else flags |= SPRITE_VFLIP;
	 }
	 
 };
 
 
 
 // --------------
 // CPlayer
 // --------------
 class CPlayer : public CSprite{
 public:
	 
	 string				name;
	 unsigned short		animInterval[2][ANIM_COUNT];
	 float				maxSpeed;
	 float				jumpStrength;
	 float				gravity;
	 float				acceleration;
	 float				deceleration;
	 
	 void				import(const string& sFilePath);
	 void				release();
	 
	 // CPlayer()
	 CPlayer(){}
	 CPlayer(const string& sFilePath){ import(sFilePath); }
	 
	 // ChangeAnimation()
	 void ChangeAnimation(int animID){
		 if(anim){
			 if(animID < 0 || animID >= ANIM_COUNT){
				 animID = 0;
			 }
			 interval[0] = animInterval[0][animID];
			 interval[1] = animInterval[1][animID];
			 currentFrame = interval[0];
		 }
	 }
	 
	 // -- Static methods --
	 static CPlayer		*getChar(string sCharName);
 };
 
 
 
 // --------------
 // CEnemy - Header for enemies
 // --------------
 class CEnemy : public CAnimation{
 public:
	 
	 string				name;
	 unsigned short		animInterval[2][EANIM_COUNT];
	 float				moveSpeed;
	 int				score;
	 
	 void				eRead(const string& sFilePath); // read enemy's information
	 
	 // CEnemy(file)
	 CEnemy(const string& sFilePath){
		 import(sFilePath);
		 eRead(sFilePath);
	 }
	 
	 // -- Static methods --
	 static CEnemy		*getEnemy(string sEnemyName);
	 
 };
 
 
 
 // --------------
 // CItem - Header for items
 // --------------
 class CItem : public CAnimation{
 public:
	 
	 string				name;
	 unsigned short		animInterval[2][IANIM_COUNT];
	 int				score;
	 
	 void				eRead(const string& sFilePath); // read item's information
	 
	 // CItem(file)
	 CItem(const string& sFilePath){
		 import(sFilePath);
		 eRead(sFilePath);
	 }
	 
	 // -- Static methods --
	 static CItem		*getItem(string sItemName);
	 
 };
 
 #endif
