/*******************************************************
 *  _____                                              *
 * |  ___|  __    __   _  __  _   __   __   ____       *
 * | |___  |  \  /  | | ||  || | \  \/  /  |__  \      *
 * |___  | |   \/   | | \/  \/ |  \    /      | |      *
 *  ___| | | /\  /\ | |   /\   |  /    \    _/ /_      *
 * |_____| |_||__||_| |__/  \__| /__/\__\  |_____|     *
 *                                                     *
 *******************************************************
 *     (c) Gaming Brasil - 2004                        *
 *     Made by Alexandre Martins (@leX)                *
 *     http://www.gamingbrasil.net                     *
 *******************************************************/
 
 #ifndef _UTILS_H
 #define _UTILS_H
 
 #include <allegro.h>
 #include <vector>
 #include <string>
 #include <stdlib.h>
 #include <time.h>
 using namespace std;
 
 // -- Utils --
 class Utils{
 public:
	 
	 // Dir/File List stuff
	 static void DirList(const string& basedir, vector<string>& v);
	 static void FileList(const string& basedir, vector<string>& v, const string filter = "*");
	 
 };
 
 // -- CEvery (logic-timer related) --
 class CEvery{
 private:
	 unsigned int	every;
	 unsigned int	counter;
	 
 public:
	 
	 void Setup(int EveryXLoops){
		 every = EveryXLoops;
		 counter = 0;
	 }
	 
	 bool MayRun(){
		 if(++counter >= every){
			 counter = 0;
			 return true;
		 }
		 return false;
	 }
	 
	 void Reset(){ counter = 0; }
	 
	 CEvery(){ Setup(0); }
	 CEvery(int x){ Setup(x); }
	 
 };
 
 // -- CTimer (real-time timer) --
 class CTimer{
 private:
	 unsigned int	every;
	 unsigned int	counter;
	 
 public:
	 
	 static unsigned int Timer(){
		 return clock()/(CLOCKS_PER_SEC/1000);
	 }
	 
	 void Setup(int EveryXMSec){
		 every = EveryXMSec;
		 counter = Timer();
	 }
	 
	 bool MayRun(){
		 if(Timer() >= counter + every){
			 counter = Timer();
			 return true;
		 }
		 return false;
	 }
	 
	 void Reset(){ counter = Timer(); }
	 
	 CTimer(){ Setup(0); }
	 CTimer(int x){ Setup(x); }
	 
 };
 
 
 // -- C2DArray --
 template<class T> class C2DArray{
 private:
	 T		**data;
 	 int	w, h;
	 void	create(int width, int height);
	 
 public:
	 
 	 void	resize(int new_width, int new_height, T value = 0);
 	 T		get(int x, int y);
	 T*		pGet(int x, int y);
 	 void	set(int x, int y, T info);
	 void	clear(T value);
	 
 	 inline int width(){ return w; }
 	 inline int height(){ return h; }
	 
 	 C2DArray(int width = 10, int height = 10){ create(width, height); }
 	 ~C2DArray();
	 
 };
 
 #include "2darray.cc"
 
 #endif
